@@ -1,6 +1,6 @@
 //
 //  Little cms
-//  Copyright (C) 1998-2001 Marti Maria
+//  Copyright (C) 1998-2003 Marti Maria
 //
 // THIS SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND,
 // EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY
@@ -28,7 +28,7 @@
 // License along with this library; if not, write to the Free Software
 // Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
-// Version 1.09d
+// Version 1.11b
 
 #ifndef __cms_H
 
@@ -41,20 +41,15 @@
 // virtually any machine.
 
 // #define USE_FLOAT        1
-#define USE_C            1
-// #define USE_ASSEMBLER    1
+// #define USE_C            1
+#define USE_ASSEMBLER    1
 
-// 3D interpolation method - Tetrahedral is slightly faster, but not always
-// is proper.  Also, tetrahedral algorithm seems it was
-// covered by a Sakamoto's patent, now expired. Code is present here only
-// for informational purposes.
+// 3D interpolation method - Tetrahedral is the default option
 
-#define USE_TRILINEAR       1
-// #define USE_TETRAHEDRAL     1
+#define USE_TETRAHEDRAL     1
+// #define USE_TRILINEAR       1
 
-
-// Define this if you are using this package as a DLL.
-// The building of DLL is now available on all windows compilers
+// Define this if you are using this package as a DLL (windows only)
 
 // #define LCMS_DLL     1
 // #define LCMS_DLL_BUILD   1
@@ -71,16 +66,10 @@
 // "long long" type This will speedup fixed point math. (USE_C only)
 #define USE_INT64        1
 
-
 // Some machines does not have a reliable 'swab' function. Usually
 // leave commented unless the testbed diagnoses the contrary.
 // #define USE_CUSTOM_SWAB   1
 
-// In conversion 16 -> 8 bps, it should be / 257, but doesn't matter
-// anyway if I just >> 8, and the speed gain is big. This toggle forces
-// the / 257 method, at a div per component cost. Don't touch unless
-// you really know what you are doing!
-// #define USE_257_FOR_16_TO_8_CONVERSION 1
 
 // ********** End of configuration toggles ******************************
 
@@ -88,6 +77,7 @@
 #include <stdlib.h>
 #include <math.h>
 #include <assert.h>
+#include <stdarg.h>
 
 // Metroworks CodeWarrior
 
@@ -112,7 +102,7 @@
 #  define USE_C               1
 #endif
 
-#if defined(__sgi__) || defined(__sgi) || defined(__powerpc__)
+#if defined(__sgi__) || defined(__sgi) || defined(__powerpc__) || defined(sparc) || defined(__ppc__)
 #   define USE_BIG_ENDIAN      1
 #endif
 
@@ -207,12 +197,62 @@ typedef HANDLE LCMSHANDLE;
 
 // Plus some additions
 
-#define lcmsSignature            ((icSignature)           0x6c636d73L)
-#define icSigLuvKData            ((icColorSpaceSignature) 0x4C75764BL)  // 'LuvK'
-#define icSigChromaticityTag     ((icTagSignature)        0x6368726dL)  // As per Addendum 2 to Spec. ICC.1:1998-09
-#define icSigChromaticityType    ((icTagTypeSignature)    0x6368726dL)
-#define icSigHexachromeData      ((icColorSpaceSignature) 0x4d434836L)  // MCH6
-#define icSigParametricCurveType ((icTagTypeSignature)    0x70617261L)  // para
+#define lcmsSignature                   ((icSignature)           0x6c636d73L)
+
+#define icSigLuvKData                   ((icColorSpaceSignature) 0x4C75764BL)  // 'LuvK'
+#define icSigHexachromeData             ((icColorSpaceSignature) 0x4d434836L)  // MCH6
+
+#define icSigChromaticityTag            ((icTagSignature)     0x6368726dL) // As per Addendum 2 to Spec. ICC.1:1998-09
+#define icSigChromaticAdaptationTag     ((icTagTypeSignature) 0x63686164L) // 'chad'
+
+#define icSigParametricCurveType        ((icTagTypeSignature) 0x70617261L)  // parametric (ICC 4.0)
+#define icSigMultiLocalizedUnicodeType  ((icTagTypeSignature) 0x6D6C7563L)
+#define icSigS15Fixed16ArrayType        ((icTagTypeSignature) 0x73663332L) 
+#define icSigChromaticityType           ((icTagTypeSignature) 0x6368726dL)
+#define icSiglutAtoBType                ((icTagTypeSignature) 0x6d414220L)  // mAB 
+#define icSiglutBtoAType                ((icTagTypeSignature) 0x6d424120L)  // mBA 
+
+
+typedef struct {
+    icUInt8Number       gridPoints[16]; // Number of grid points in each dimension.  
+    icUInt8Number       prec;           // Precision of data elements in bytes.      
+    icUInt8Number       pad1;
+    icUInt8Number       pad2;
+    icUInt8Number       pad3;
+    /*icUInt8Number     data[icAny];     Data follows see spec for size */
+} icCLutStruct;
+
+// icLutAtoB  
+typedef struct {
+    icUInt8Number       inputChan;      // Number of input channels     
+    icUInt8Number       outputChan;     // Number of output channels    
+    icUInt8Number       pad1;
+    icUInt8Number       pad2;
+    icUInt32Number      offsetB;        // Offset to first "B" curve    
+    icUInt32Number      offsetMat;      // Offset to matrix             
+    icUInt32Number      offsetM;        // Offset to first "M" curve    
+    icUInt32Number      offsetC;        // Offset to CLUT               
+    icUInt32Number      offsetA;        // Offset to first "A" curve    
+    /*icUInt8Number     data[icAny];     Data follows see spec for size */
+} icLutAtoB;
+
+// icLutBtoA  
+typedef struct {
+    icUInt8Number       inputChan;      // Number of input channels     
+    icUInt8Number       outputChan;     // Number of output channels    
+    icUInt8Number       pad1;
+    icUInt8Number       pad2;
+    icUInt32Number      offsetB;        // Offset to first "B" curve    
+    icUInt32Number      offsetMat;      // Offset to matrix             
+    icUInt32Number      offsetM;        // Offset to first "M" curve    
+    icUInt32Number      offsetC;        // Offset to CLUT               
+    icUInt32Number      offsetA;        // Offset to first "A" curve    
+    /*icUInt8Number     data[icAny];     Data follows see spec for size */
+} icLutBtoA;
+
+
+
+
 
 #ifdef __cplusplus
 extern "C" {
@@ -296,9 +336,9 @@ extern "C" {
 typedef LCMSHANDLE cmsHPROFILE;        // Opaque typedefs to hide internals
 typedef LCMSHANDLE cmsHTRANSFORM;
 
-// Format of pixel is defined by one integer, using bits as follows
+// Format of pixel is defined by one DWORD, using bit fields as follows
 //
-//            TTTTT - Y F P X S EEE CCCC BBB
+//            TTTTT U Y F P X S EEE CCCC BBB
 //
 //            T: Pixeltype
 //            F: Flavor  0=MinIsBlack(Chocolate) 1=MinIsWhite(Vanilla)
@@ -309,7 +349,6 @@ typedef LCMSHANDLE cmsHTRANSFORM;
 //            C: Channels (Samples per pixel)
 //            B: Bytes per sample
 //            Y: Swap first - changes ABGR to BGRA and KCMY to CMYK
-//            -: Unused (reserved)
 
 
 #define COLORSPACE_SH(s)       ((s) << 16)
@@ -325,8 +364,7 @@ typedef LCMSHANDLE cmsHTRANSFORM;
 // Pixel types
 
 #define PT_ANY       0    // Don't check colorspace
-
-                           // 1 & 2 are reserved
+                          // 1 & 2 are reserved
 #define PT_GRAY      3
 #define PT_RGB       4
 #define PT_CMY       5
@@ -340,6 +378,7 @@ typedef LCMSHANDLE cmsHTRANSFORM;
 #define PT_HLS       13
 #define PT_Yxy       14
 #define PT_HiFi      15
+#define PT_HiFi8     16
 
 #define NOCOLORSPACECHECK(x)    ((x) & 0xFFFF)
 
@@ -469,6 +508,7 @@ typedef LCMSHANDLE cmsHTRANSFORM;
 
 #define TYPE_XYZ_16            (COLORSPACE_SH(PT_XYZ)|CHANNELS_SH(3)|BYTES_SH(2))
 #define TYPE_Lab_8             (COLORSPACE_SH(PT_Lab)|CHANNELS_SH(3)|BYTES_SH(1))
+#define TYPE_ALab_8            (COLORSPACE_SH(PT_Lab)|CHANNELS_SH(3)|BYTES_SH(1)|EXTRA_SH(1)|DOSWAP_SH(1))
 #define TYPE_Lab_16            (COLORSPACE_SH(PT_Lab)|CHANNELS_SH(3)|BYTES_SH(2))
 #define TYPE_Yxy_16            (COLORSPACE_SH(PT_Yxy)|CHANNELS_SH(3)|BYTES_SH(2))
 
@@ -505,6 +545,18 @@ typedef LCMSHANDLE cmsHTRANSFORM;
 #define TYPE_HSV_16_PLANAR   (COLORSPACE_SH(PT_HSV)|CHANNELS_SH(3)|BYTES_SH(2)|PLANAR_SH(1))
 #define TYPE_HSV_16_SE       (COLORSPACE_SH(PT_HSV)|CHANNELS_SH(3)|BYTES_SH(2)|ENDIAN16_SH(1))
 
+// Named color index. Only 16 bits allowed (don't check colorspace) 
+
+#define TYPE_NAMED_COLOR_INDEX   (CHANNELS_SH(1)|BYTES_SH(2))
+
+// Double values. Painful slow, but sometimes helpful. NOTE THAT BYTES IS SET TO ZERO!
+
+#define TYPE_XYZ_DBL        (COLORSPACE_SH(PT_XYZ)|CHANNELS_SH(3)|BYTES_SH(0))
+#define TYPE_Lab_DBL        (COLORSPACE_SH(PT_Lab)|CHANNELS_SH(3)|BYTES_SH(0))
+#define TYPE_GRAY_DBL       (COLORSPACE_SH(PT_GRAY)|CHANNELS_SH(1)|BYTES_SH(0))
+#define TYPE_RGB_DBL        (COLORSPACE_SH(PT_RGB)|CHANNELS_SH(3)|BYTES_SH(0))
+#define TYPE_CMYK_DBL       (COLORSPACE_SH(PT_CMYK)|CHANNELS_SH(4)|BYTES_SH(0))
+
 #endif
 
 // Gamma tables.
@@ -514,7 +566,10 @@ typedef struct {
     int  nEntries;
     WORD GammaTable[1];
 
-    } GAMMATABLE, FAR* LPGAMMATABLE;
+    } GAMMATABLE;
+
+typedef GAMMATABLE FAR* LPGAMMATABLE;
+
 
 // Sampled curves (1D)
 
@@ -523,7 +578,10 @@ typedef struct {
     int     nItems;
     double* Values;
 
-    } SAMPLEDCURVE, FAR* LPSAMPLEDCURVE;
+    } SAMPLEDCURVE;
+
+typedef SAMPLEDCURVE FAR* LPSAMPLEDCURVE;
+
 
 
 // Vectors
@@ -532,14 +590,19 @@ typedef struct {                // Float Vector
 
     double n[3];
 
-    } VEC3, FAR* LPVEC3;
+    } VEC3;
+
+typedef VEC3 FAR* LPVEC3;
+
 
 
 typedef struct {                // Matrix
         
     VEC3 v[3];
     
-    } MAT3, FAR* LPMAT3;
+    } MAT3; 
+
+typedef MAT3 FAR* LPMAT3;
 
 
 // Colorspace values
@@ -550,7 +613,9 @@ typedef struct {
         double Y;
         double Z;
 
-    } cmsCIEXYZ, FAR* LPcmsCIEXYZ;
+    } cmsCIEXYZ; 
+        
+typedef cmsCIEXYZ FAR* LPcmsCIEXYZ;
 
 typedef struct {
                
@@ -558,7 +623,9 @@ typedef struct {
         double y;
         double Y;
 
-    } cmsCIExyY, FAR* LPcmsCIExyY;
+    } cmsCIExyY;
+
+typedef cmsCIExyY FAR* LPcmsCIExyY;
 
 typedef struct {
                
@@ -566,7 +633,9 @@ typedef struct {
         double a;               
         double b;
                
-    } cmsCIELab,FAR* LPcmsCIELab;
+    } cmsCIELab;
+
+typedef cmsCIELab FAR* LPcmsCIELab;
 
 typedef struct {
                
@@ -574,7 +643,9 @@ typedef struct {
         double C;
         double h;
 
-    } cmsCIELCh,FAR* LPcmsCIELCh;
+    } cmsCIELCh;
+
+typedef cmsCIELCh FAR* LPcmsCIELCh;
 
 typedef struct {
                
@@ -582,7 +653,9 @@ typedef struct {
         double C;
         double h;
 
-    } cmsJCh, FAR* LPcmsJCh;
+    } cmsJCh;
+
+typedef cmsJCh FAR* LPcmsJCh;
 
 // Primaries
 
@@ -592,64 +665,79 @@ typedef struct {
         cmsCIEXYZ  Green;
         cmsCIEXYZ  Blue;
 
-    } cmsCIEXYZTRIPLE, FAR* LPcmsCIEXYZTRIPLE;
+    } cmsCIEXYZTRIPLE;
+
+typedef cmsCIEXYZTRIPLE FAR* LPcmsCIEXYZTRIPLE;
 
 
 typedef struct {
-
               
         cmsCIExyY  Red;
         cmsCIExyY  Green;
         cmsCIExyY  Blue;
 
-    } cmsCIExyYTRIPLE, FAR* LPcmsCIExyYTRIPLE;
+    } cmsCIExyYTRIPLE;
+
+typedef cmsCIExyYTRIPLE FAR* LPcmsCIExyYTRIPLE;
 
 
 
-#define D50X  (0.9642)  // Following ICC spec
+// Following ICC spec
+
+#define D50X  (0.9642)  
 #define D50Y  (1.0)
 #define D50Z  (0.8249)
 
 // Does return pointers to constant structs
+
 LCMSAPI LPcmsCIEXYZ LCMSEXPORT cmsD50_XYZ(void);
 LCMSAPI LPcmsCIExyY LCMSEXPORT cmsD50_xyY(void); 
 
+
 // Input/Output
 
 LCMSAPI cmsHPROFILE   LCMSEXPORT cmsOpenProfileFromFile(const char *ICCProfile, const char *sAccess);
 LCMSAPI cmsHPROFILE   LCMSEXPORT cmsOpenProfileFromMem(LPVOID MemPtr, DWORD dwSize);
+LCMSAPI BOOL          LCMSEXPORT cmsCloseProfile(cmsHPROFILE hProfile);
+
+// Predefined run-time profiles
 
 LCMSAPI cmsHPROFILE   LCMSEXPORT cmsCreateRGBProfile(LPcmsCIExyY WhitePoint,
                                         LPcmsCIExyYTRIPLE Primaries,
                                         LPGAMMATABLE TransferFunction[3]);
 
-// Predefined profiles
+LCMSAPI cmsHPROFILE   LCMSEXPORT cmsCreateGrayProfile(LPcmsCIExyY WhitePoint,
+                                              LPGAMMATABLE TransferFunction);
+
+LCMSAPI cmsHPROFILE LCMSEXPORT cmsCreateLinearizationDeviceLink(icColorSpaceSignature ColorSpace,
+                                                        LPGAMMATABLE TransferFunctions[]);
+
+LCMSAPI cmsHPROFILE LCMSEXPORT cmsCreateInkLimitingDeviceLink(icColorSpaceSignature ColorSpace,
+                                                      double Limit);
+
 
 LCMSAPI cmsHPROFILE   LCMSEXPORT cmsCreateLabProfile(LPcmsCIExyY WhitePoint);
 LCMSAPI cmsHPROFILE   LCMSEXPORT cmsCreateXYZProfile(void);
 LCMSAPI cmsHPROFILE   LCMSEXPORT cmsCreate_sRGBProfile(void);
 
-LCMSAPI BOOL          LCMSEXPORT cmsCloseProfile(cmsHPROFILE hProfile);
 
-// Utils
+// Colorimetric space conversions
 
 LCMSAPI void          LCMSEXPORT cmsXYZ2xyY(LPcmsCIExyY Dest, const LPcmsCIEXYZ Source);
 LCMSAPI void          LCMSEXPORT cmsxyY2XYZ(LPcmsCIEXYZ Dest, const LPcmsCIExyY Source);
 LCMSAPI void          LCMSEXPORT cmsXYZ2Lab(LPcmsCIEXYZ WhitePoint, LPcmsCIELab Lab, const LPcmsCIEXYZ xyz);
 LCMSAPI void          LCMSEXPORT cmsLab2XYZ(LPcmsCIEXYZ WhitePoint, LPcmsCIEXYZ xyz, const LPcmsCIELab Lab);
-LCMSAPI void          LCMSEXPORT cmsLabEncoded2Float(LPcmsCIELab Lab, const WORD wLab[3]);
-LCMSAPI void          LCMSEXPORT cmsFloat2LabEncoded(WORD wLab[3], const LPcmsCIELab Lab);
-LCMSAPI void          LCMSEXPORT cmsXYZEncoded2Float(LPcmsCIEXYZ fxyz, const WORD XYZ[3]);
-LCMSAPI void          LCMSEXPORT cmsFloat2XYZEncoded(WORD XYZ[3], const LPcmsCIEXYZ fXYZ);
 LCMSAPI void          LCMSEXPORT cmsLab2LCh(LPcmsCIELCh LCh, const LPcmsCIELab Lab);
 LCMSAPI void          LCMSEXPORT cmsLCh2Lab(LPcmsCIELab Lab, const LPcmsCIELCh LCh);
 
+
 // CIELab handling
 
 LCMSAPI double        LCMSEXPORT cmsDeltaE(LPcmsCIELab Lab1, LPcmsCIELab Lab2);
 LCMSAPI double        LCMSEXPORT cmsCIE94DeltaE(LPcmsCIELab Lab1, LPcmsCIELab Lab2);
 LCMSAPI double        LCMSEXPORT cmsBFDdeltaE(LPcmsCIELab Lab1, LPcmsCIELab Lab2);
 LCMSAPI double        LCMSEXPORT cmsCMCdeltaE(LPcmsCIELab Lab1, LPcmsCIELab Lab2);
+LCMSAPI double        LCMSEXPORT cmsCIE2000DeltaE(LPcmsCIELab Lab1, LPcmsCIELab Lab2, double Kl, double Kc, double Kh);
 
 LCMSAPI void          LCMSEXPORT cmsClampLab(LPcmsCIELab Lab, double amax, double amin, double bmax, double bmin);
 
@@ -683,7 +771,9 @@ typedef struct {
               int       surround;
               double    D_value;
 
-              } cmsViewingConditions, FAR* LPcmsViewingConditions;
+    } cmsViewingConditions;
+
+typedef cmsViewingConditions FAR* LPcmsViewingConditions;
 
 
 LCMSAPI LCMSHANDLE    LCMSEXPORT cmsCIECAM97sInit(LPcmsViewingConditions pVC2);
@@ -691,6 +781,15 @@ LCMSAPI void          LCMSEXPORT cmsCIECAM97sDone(LCMSHANDLE hModel);
 LCMSAPI void          LCMSEXPORT cmsCIECAM97sForward(LCMSHANDLE hModel, LPcmsCIEXYZ pIn, LPcmsJCh pOut);
 LCMSAPI void          LCMSEXPORT cmsCIECAM97sReverse(LCMSHANDLE hModel, LPcmsJCh pIn,    LPcmsCIEXYZ pOut);
 
+
+// CIECAM02
+
+LCMSAPI LCMSHANDLE LCMSEXPORT cmsCIECAM02Init(LPcmsViewingConditions pVC);
+LCMSAPI void       LCMSEXPORT cmsCIECAM02Done(LCMSHANDLE hModel);
+LCMSAPI void       LCMSEXPORT cmsCIECAM02Forward(LCMSHANDLE hModel, LPcmsCIEXYZ pIn, LPcmsJCh pOut);
+LCMSAPI void       LCMSEXPORT cmsCIECAM02Reverse(LCMSHANDLE hModel, LPcmsJCh pIn,    LPcmsCIEXYZ pOut);
+
+
 // Gamma
 
 LCMSAPI LPGAMMATABLE  LCMSEXPORT cmsBuildGamma(int nEntries, double Gamma);
@@ -698,14 +797,22 @@ LCMSAPI LPGAMMATABLE  LCMSEXPORT cmsBuildParametricGamma(int nEntries, int Type,
 LCMSAPI LPGAMMATABLE  LCMSEXPORT cmsAllocGamma(int nEntries);
 LCMSAPI void          LCMSEXPORT cmsFreeGamma(LPGAMMATABLE Gamma);
 LCMSAPI void          LCMSEXPORT cmsFreeGammaTriple(LPGAMMATABLE Gamma[3]);
+LCMSAPI LPGAMMATABLE  LCMSEXPORT cmsDupGamma(LPGAMMATABLE Src);
 LCMSAPI LPGAMMATABLE  LCMSEXPORT cmsReverseGamma(int nResultSamples, LPGAMMATABLE InGamma);
 LCMSAPI LPGAMMATABLE  LCMSEXPORT cmsJoinGamma(LPGAMMATABLE InGamma,  LPGAMMATABLE OutGamma);
 LCMSAPI LPGAMMATABLE  LCMSEXPORT cmsJoinGammaEx(LPGAMMATABLE InGamma,  LPGAMMATABLE OutGamma, int nPoints);
 LCMSAPI BOOL          LCMSEXPORT cmsSmoothGamma(LPGAMMATABLE Tab, double lambda);
 LCMSAPI double        LCMSEXPORT cmsEstimateGamma(LPGAMMATABLE t);
+LCMSAPI double        LCMSEXPORT cmsEstimateGammaEx(LPWORD Table, int nEntries, double Thereshold); 
+LCMSAPI LPGAMMATABLE  LCMSEXPORT cmsReadICCGamma(cmsHPROFILE hProfile, icTagSignature sig);
+LCMSAPI LPGAMMATABLE  LCMSEXPORT cmsReadICCGammaReversed(cmsHPROFILE hProfile, icTagSignature sig);
+
+
+
 
 // Access to Profile data.
 
+LCMSAPI void          LCMSEXPORT cmsSetLanguage(int LanguageCode, int CountryCode);
 LCMSAPI BOOL          LCMSEXPORT cmsTakeMediaWhitePoint(LPcmsCIEXYZ Dest, cmsHPROFILE hProfile);
 LCMSAPI BOOL          LCMSEXPORT cmsTakeMediaBlackPoint(LPcmsCIEXYZ Dest, cmsHPROFILE hProfile);
 LCMSAPI BOOL          LCMSEXPORT cmsTakeIluminant(LPcmsCIEXYZ Dest, cmsHPROFILE hProfile);
@@ -718,7 +825,7 @@ LCMSAPI int           LCMSEXPORT cmsTakeRenderingIntent(cmsHPROFILE hProfile);
 
 LCMSAPI BOOL          LCMSEXPORT cmsTakeCharTargetData(cmsHPROFILE hProfile, char** Data, size_t* len);
                                                    
-// Translate form our notation to ICC one
+// Translate form our notation to ICC 
 LCMSAPI icColorSpaceSignature LCMSEXPORT _cmsICCcolorSpace(int OurNotation);
 LCMSAPI                   int LCMSEXPORT _cmsChannelsOf(icColorSpaceSignature ColorSpace);
 
@@ -726,7 +833,7 @@ LCMSAPI                   int LCMSEXPORT _cmsChannelsOf(icColorSpaceSignature Co
 #define LCMS_USED_AS_OUTPUT     1
 #define LCMS_USED_AS_PROOF      2
 
-LCMSAPI BOOL          LCMSEXPORT cmsIsIntentSupported(cmsHPROFILE hProfile, int Intent, int UsedDirection);
+LCMSAPI BOOL         LCMSEXPORT cmsIsIntentSupported(cmsHPROFILE hProfile, int Intent, int UsedDirection);
 
 LCMSAPI icColorSpaceSignature   LCMSEXPORT cmsGetPCS(cmsHPROFILE hProfile);
 LCMSAPI icColorSpaceSignature   LCMSEXPORT cmsGetColorSpace(cmsHPROFILE hProfile);
@@ -736,6 +843,8 @@ LCMSAPI icProfileClassSignature LCMSEXPORT cmsGetDeviceClass(cmsHPROFILE hProfil
 LCMSAPI void          LCMSEXPORT cmsSetDeviceClass(cmsHPROFILE hProfile, icProfileClassSignature sig);
 LCMSAPI void          LCMSEXPORT cmsSetColorSpace(cmsHPROFILE hProfile, icColorSpaceSignature sig);
 LCMSAPI void          LCMSEXPORT cmsSetPCS(cmsHPROFILE hProfile, icColorSpaceSignature pcs);
+LCMSAPI void          LCMSEXPORT cmsSetRenderingIntent(cmsHPROFILE hProfile, int RenderingIntent);
+
 
 // Intents
 
@@ -749,12 +858,18 @@ LCMSAPI void          LCMSEXPORT cmsSetPCS(cmsHPROFILE hProfile, icColorSpaceSig
 #define cmsFLAGS_MATRIXINPUT              0x0001
 #define cmsFLAGS_MATRIXOUTPUT             0x0002
 #define cmsFLAGS_MATRIXONLY               (cmsFLAGS_MATRIXINPUT|cmsFLAGS_MATRIXOUTPUT)
-#define cmsFLAGS_NOTPRECALC               0x0100
+
+#define cmsFLAGS_NOPRELINEARIZATION       0x0010    // Don't create prelinearization tables
+                                                    // on precalculated transforms (internal use)                   
+
+#define cmsFLAGS_NOTPRECALC               0x0100    
 #define cmsFLAGS_NULLTRANSFORM            0x0200    // Don't transform anyway
 #define cmsFLAGS_HIGHRESPRECALC           0x0400    // Use more memory to give better accurancy
+#define cmsFLAGS_LOWRESPRECALC            0x0800    // Use less memory to minimize resouces
 
 
-#define cmsFLAGS_WHITEBLACKCOMPENSATION   0x2000    // Matches black and white on precalculated transforms
+#define cmsFLAGS_WHITEBLACKCOMPENSATION   0x2000    
+#define cmsFLAGS_BLACKPOINTCOMPENSATION   cmsFLAGS_WHITEBLACKCOMPENSATION   
 
 // Proofing flags
 
@@ -763,6 +878,7 @@ LCMSAPI void          LCMSEXPORT cmsSetPCS(cmsHPROFILE hProfile, icColorSpaceSig
 
 
 
+
 // Transforms
 
 LCMSAPI cmsHTRANSFORM LCMSEXPORT cmsCreateTransform(cmsHPROFILE Input,
@@ -795,13 +911,39 @@ LCMSAPI void         LCMSEXPORT cmsDoTransform(cmsHTRANSFORM Transform,
                                                  LPVOID OutputBuffer,
                                                  unsigned int Size);
 
+LCMSAPI void         LCMSEXPORT cmsChangeBuffersFormat(cmsHTRANSFORM hTransform, DWORD InputFormat, DWORD dwOutputFormat);
+
 LCMSAPI void         LCMSEXPORT cmsSetAlarmCodes(int r, int g, int b);
 LCMSAPI void         LCMSEXPORT cmsGetAlarmCodes(int *r, int *g, int *b);
 
-// Profile creation - New in 1.09
+
+// Named color support
+
+LCMSAPI int  LCMSEXPORT cmsNamedColorCount(cmsHTRANSFORM xform);
+LCMSAPI BOOL LCMSEXPORT cmsNamedColorInfo(cmsHTRANSFORM xform, int nColor, char* Name, char* Prefix, char* Suffix);
+LCMSAPI int  LCMSEXPORT cmsNamedColorIndex(cmsHTRANSFORM xform, const char* Name);
+
+// Profile creation 
 
 LCMSAPI BOOL LCMSEXPORT cmsAddTag(cmsHPROFILE hProfile, icTagSignature sig, void* data);
 
+// Converts a transform to a devicelink profile
+LCMSAPI cmsHPROFILE LCMSEXPORT cmsTransform2DeviceLink(cmsHTRANSFORM hTransform, DWORD dwFlags);
+
+
+// Save profile
+LCMSAPI BOOL LCMSEXPORT _cmsSaveProfile(cmsHPROFILE hProfile, const char* FileName);
+LCMSAPI BOOL LCMSEXPORT _cmsSaveProfileToMem(cmsHPROFILE hProfile, void *MemPtr, 
+                                                                size_t* BytesNeeded);
+
+
+
+// PostScript ColorRenderingDictionary and ColorSpaceArray
+
+LCMSAPI DWORD LCMSEXPORT cmsGetPostScriptCSA(cmsHPROFILE hProfile, int Intent, LPVOID Buffer, DWORD dwBufferLen);
+LCMSAPI DWORD LCMSEXPORT cmsGetPostScriptCRD(cmsHPROFILE hProfile, int Intent, LPVOID Buffer, DWORD dwBufferLen);
+
+
 // Error handling
 
 #define LCMS_ERROR_ABORT    0
@@ -814,20 +956,27 @@ LCMSAPI int LCMSEXPORT cmsErrorAction(int nAction);
 #define LCMS_ERRC_RECOVERABLE    0x2000
 #define LCMS_ERRC_ABORTED        0x3000
 
+typedef int (* cmsErrorHandlerFunction)(int ErrorCode, const char *ErrorText);
+
 void cdecl cmsSignalError(int ErrorCode, const char *ErrorText, ...);
 
+LCMSAPI void LCMSEXPORT cmsSetErrorHandler(cmsErrorHandlerFunction Fn);
+
 
 // LUT manipulation
 
+
 typedef struct _lcms_LUT_struc LUT, FAR* LPLUT; // opaque pointer
 
 LCMSAPI LPLUT LCMSEXPORT cmsAllocLUT(void);
 LCMSAPI LPLUT LCMSEXPORT cmsAllocLinearTable(LPLUT NewLUT, LPGAMMATABLE Tables[], int nTable);
 LCMSAPI LPLUT LCMSEXPORT cmsAlloc3DGrid(LPLUT Lut, int clutPoints, int inputChan, int outputChan);
 LCMSAPI LPLUT LCMSEXPORT cmsSetMatrixLUT(LPLUT Lut, LPMAT3 M);
+LCMSAPI LPLUT LCMSEXPORT cmsSetMatrixLUT4(LPLUT Lut, LPMAT3 M, LPVEC3 off, DWORD dwFlags);
 LCMSAPI void  LCMSEXPORT cmsFreeLUT(LPLUT Lut);
 LCMSAPI void  LCMSEXPORT cmsEvalLUT(LPLUT Lut, WORD In[], WORD Out[]);
 LCMSAPI LPLUT LCMSEXPORT cmsReadICCLut(cmsHPROFILE hProfile, icTagSignature sig);
+LCMSAPI LPLUT LCMSEXPORT cmsDupLUT(LPLUT Orig);
 
 // LUT Sampling
 
@@ -835,13 +984,43 @@ typedef int (* _cmsSAMPLER)(register WORD In[],
                             register WORD Out[],
                             register LPVOID Cargo);
 
+#define SAMPLER_HASTL1      LUT_HASTL1
+#define SAMPLER_HASTL2      LUT_HASTL2
+#define SAMPLER_INSPECT     0x01000000
+
 LCMSAPI int LCMSEXPORT cmsSample3DGrid(LPLUT Lut, _cmsSAMPLER Sampler, LPVOID Cargo, DWORD dwFlags);
 
+// Formatters
+
+typedef unsigned char* (* cmsFORMATTER)(register void* CMMcargo,
+                                        register WORD ToUnroll[],
+                                        register LPBYTE Buffer);
+
+LCMSAPI void LCMSEXPORT cmsSetUserFormatters(cmsHTRANSFORM hTransform, DWORD dwInput,  cmsFORMATTER Input,
+                                                               DWORD dwOutput, cmsFORMATTER Output);
+
+LCMSAPI void LCMSEXPORT cmsGetUserFormatters(cmsHTRANSFORM hTransform, 
+                                                               LPDWORD InputFormat, cmsFORMATTER* Input, 
+                                                               LPDWORD OutputFormat, cmsFORMATTER* Output);
+
+
 // ***************************************************************************
 // End of Little cms API From here functions are private
 // You can use them only if using static libraries, and at your own risk of
 // be stripped or changed at futures releases.
 
+#ifndef LCMS_APIONLY
+
+
+// Compatibility with anterior versions-- not needed anymore
+//  -- Morge
+
+LCMSAPI void          LCMSEXPORT cmsLabEncoded2Float(LPcmsCIELab Lab, const WORD wLab[3]);
+LCMSAPI void          LCMSEXPORT cmsFloat2LabEncoded(WORD wLab[3], const LPcmsCIELab Lab);
+LCMSAPI void          LCMSEXPORT cmsXYZEncoded2Float(LPcmsCIEXYZ fxyz, const WORD XYZ[3]);
+LCMSAPI void          LCMSEXPORT cmsFloat2XYZEncoded(WORD XYZ[3], const LPcmsCIEXYZ fXYZ);
+
+
 // Profiling Extensions --- Would be removed from API in future revisions
 
 LCMSAPI BOOL LCMSEXPORT _cmsAddTextTag(cmsHPROFILE hProfile,  icTagSignature sig, const char* Text);
@@ -850,7 +1029,7 @@ LCMSAPI BOOL LCMSEXPORT _cmsAddLUTTag(cmsHPROFILE hProfile,   icTagSignature sig
 LCMSAPI BOOL LCMSEXPORT _cmsAddGammaTag(cmsHPROFILE hProfile, icTagSignature sig, LPGAMMATABLE TransferFunction);
 LCMSAPI BOOL LCMSEXPORT _cmsAddChromaticityTag(cmsHPROFILE hProfile, icTagSignature sig, LPcmsCIExyYTRIPLE Chrm);
 
-LCMSAPI BOOL LCMSEXPORT _cmsSaveProfile(cmsHPROFILE hProfile, const char* FileName);
+//  -- end of morge
 
 
 // Alignment handling (needed in ReadLUT16 and ReadLUT8)
@@ -921,6 +1100,8 @@ void cdecl VEC3perK(LPVEC3 r, LPVEC3 v, double d);
 void cdecl VEC3minus(LPVEC3 r, LPVEC3 a, LPVEC3 b);
 void cdecl VEC3perComp(LPVEC3 r, LPVEC3 a, LPVEC3 b);
 BOOL cdecl VEC3equal(LPWVEC3 a, LPWVEC3 b, double Tolerance);
+void cdecl VEC3scaleAndCut(LPWVEC3 r, LPVEC3 v, double d);
+void cdecl VEC3saturate(LPVEC3 v);
 
 void cdecl MAT3identity(LPMAT3 a);
 void cdecl MAT3per(LPMAT3 r, LPMAT3 a, LPMAT3 b);
@@ -931,7 +1112,7 @@ void cdecl MAT3toFix(LPWMAT3 r, LPMAT3 v);
 void cdecl MAT3fromFix(LPMAT3 r, LPWMAT3 v);
 void cdecl MAT3evalW(LPWVEC3 r, LPWMAT3 a, LPWVEC3 v);
 BOOL cdecl MAT3isIdentity(LPWMAT3 a, double Tolerance);
-
+void cdecl MAT3scaleAndCut(LPWMAT3 r, LPMAT3 v, double d);
 
 // Is a table linear?
 
@@ -940,8 +1121,15 @@ int  cdecl cmsIsLinear(WORD Table[], int nEntries);
 // I hold this structures describing domain
 // details mainly for optimization purposes.
 
+struct _lcms_l16params_struc;
 
-typedef struct {              // Used on 16 bits interpolations
+typedef void (* _cms3DLERP)(WORD Input[],
+                            WORD Output[],
+                            WORD LutTable[],
+                            struct _lcms_l16params_struc* p);
+
+
+typedef struct _lcms_l16params_struc {    // Used on 16 bits interpolations
 
                int nSamples;       // Valid on all kinds of tables
                int nInputs;        // != 1 only in 3D interpolation
@@ -952,7 +1140,9 @@ typedef struct {              // Used on 16 bits interpolations
                int opta1, opta2;
                int opta3, opta4;     // Optimization for 3D LUT
                int opta5, opta6;
+               int opta7, opta8;
 
+               _cms3DLERP Interp3D;
 
                } L16PARAMS, *LPL16PARAMS;
 
@@ -981,14 +1171,11 @@ void cdecl cmsTetrahedralInterp16(WORD Input[],
 
 // New in rev 4.0 of ICC spec
 
-#define LUT_HASMATRIX2     0x0020   // Matrix for LutAToB
-#define LUT_HASMATRIX2OFS  0x0040   // Offset for LutAToB
+#define LUT_HASMATRIX3     0x0020   // Matrix + offset for LutAToB
+#define LUT_HASMATRIX4     0x0040   // Matrix + offset for LutBToA
 
-#define LUT_HASTL3         0x0080   // 'M' curves for LutAToB
-#define LUT_HASTL4         0x0100   // 'M' curves for LutBToA
-
-#define LUT_HASMATRIX3     0x0200   // Matrix for LutBToA
-#define LUT_HASMATRIX3OFS  0x0400   // Offset for LutBToA
+#define LUT_HASTL3         0x0100   // '3' curves for LutAToB
+#define LUT_HASTL4         0x0200   // '4' curves for LutBToA
 
 
 
@@ -1010,6 +1197,7 @@ struct _lcms_LUT_struc {
                LPWORD L2[MAXCHANNELS];          // Last linearization
 
                LPWORD T;                        // 3D CLUT
+               unsigned int Tsize;              // CLUT size in bytes
 
               // Parameters & Optimizations
 
@@ -1023,28 +1211,21 @@ struct _lcms_LUT_struc {
 
                WMAT3 Mat3;
                WVEC3 Ofs3;
-               LPWORD L3[MAXCHANNELS];
-               LPWORD L4[MAXCHANNELS];
+               LPWORD L3[MAXCHANNELS]; 
+               L16PARAMS L3params;  
+               unsigned int L3Entries;
+
                WMAT3 Mat4;
                WVEC3 Ofs4;
-
-               L16PARAMS L3params;
-               L16PARAMS L4params;
-
-               unsigned int L3Entries;
+               LPWORD L4[MAXCHANNELS];                           
+               L16PARAMS L4params;             
                unsigned int L4Entries;
 
                }; // LUT, FAR* LPLUT;
 
 
-
-
-
-// Gamma
-LPGAMMATABLE cdecl cmsScaleGamma(LPGAMMATABLE Input, Fixed32 Factor);
 BOOL         cdecl _cmsSmoothEndpoints(LPWORD Table, int nEntries);
 
-
 // Sampled curves
 
 LPSAMPLEDCURVE cdecl cmsAllocSampledCurve(int nItems);
@@ -1067,6 +1248,7 @@ LPSAMPLEDCURVE cdecl cmsJoinSampledCurves(LPSAMPLEDCURVE X, LPSAMPLEDCURVE Y, in
 #define MATSHAPER_HASSHAPER        0x0002
 #define MATSHAPER_INPUT            0x0004        // Behaviour
 #define MATSHAPER_OUTPUT           0x0008
+#define MATSHAPER_HASINPSHAPER     0x0010
 #define MATSHAPER_ALLSMELTED       (MATSHAPER_INPUT|MATSHAPER_OUTPUT)
 
 
@@ -1074,25 +1256,35 @@ typedef struct {
                DWORD dwFlags;
 
                WMAT3 Matrix;
-               L16PARAMS p16;
 
+               L16PARAMS p16;       // Primary curve
                LPWORD L[3];
+               
+               L16PARAMS p2_16;     // Secondary curve (used as input in smelted ones)
+               LPWORD L2[3];
 
                } MATSHAPER, FAR* LPMATSHAPER;
 
 LPMATSHAPER cdecl cmsAllocMatShaper(LPMAT3 matrix, LPGAMMATABLE Shaper[], DWORD Behaviour);
-LPMATSHAPER cdecl cmsAllocMonoMatShaper(LPGAMMATABLE Tables[], DWORD Behaviour);
+LPMATSHAPER cdecl cmsAllocMatShaper2(LPMAT3 matrix, LPGAMMATABLE In[], LPGAMMATABLE Out[], DWORD Behaviour);
+
 void        cdecl cmsFreeMatShaper(LPMATSHAPER MatShaper);
 void        cdecl cmsEvalMatShaper(LPMATSHAPER MatShaper, WORD In[], WORD Out[]);
 
-LPGAMMATABLE cdecl cmsReadICCGamma(cmsHPROFILE hProfile, icTagSignature sig);
 BOOL         cdecl cmsReadICCMatrixRGB2XYZ(LPMAT3 r, cmsHPROFILE hProfile);
 
+LPMATSHAPER  cdecl cmsBuildInputMatrixShaper(cmsHPROFILE InputProfile, LPDWORD dwFlags);
+LPMATSHAPER  cdecl cmsBuildOutputMatrixShaper(cmsHPROFILE OutputProfile, LPDWORD dwFlags);
+
+
+
 // White Point & Primary chromas handling
-BOOL cdecl cmsAdaptationMatrix(LPMAT3 r, LPcmsCIEXYZ FromIll, LPcmsCIEXYZ ToIll);
+BOOL cdecl cmsAdaptationMatrix(LPMAT3 r, LPMAT3 ConeMatrix, LPcmsCIEXYZ FromIll, LPcmsCIEXYZ ToIll);
 BOOL cdecl cmsAdaptMatrixToD50(LPMAT3 r, LPcmsCIExyY SourceWhitePt);
 BOOL cdecl cmsAdaptMatrixFromD50(LPMAT3 r, LPcmsCIExyY DestWhitePt);
 
+BOOL cdecl cmsReadChromaticAdaptationMatrix(LPMAT3 r, cmsHPROFILE hProfile);
+
 // Inter-PCS conversion routines. They assume D50 as white point.
 void cdecl cmsXYZ2LabEncoded(WORD XYZ[3], WORD Lab[3]);
 void cdecl cmsLab2XYZEncoded(WORD Lab[3], WORD XYZ[3]);
@@ -1103,6 +1295,92 @@ void cdecl _cmsIdentifyWhitePoint(char *Buffer, LPcmsCIEXYZ WhitePt);
 // Quantize to WORD in a (MaxSamples - 1) domain
 WORD cdecl _cmsQuantizeVal(double i, int MaxSamples);
 
+// Named color support
+typedef struct {                
+                char Name[MAX_PATH];
+                WORD PCS[3];
+                WORD DeviceColorant[MAXCHANNELS];
+                
+
+        } cmsNAMEDCOLOR, FAR* LPcmsNAMEDCOLOR;
+
+typedef struct {
+                int nColors;                
+                int Allocated;
+                int ColorantCount;              
+                LPcmsNAMEDCOLOR List;
+                char Prefix[33];
+                char Suffix[33];
+
+        } cmsNAMEDCOLORLIST, FAR* LPcmsNAMEDCOLORLIST;
+
+LPcmsNAMEDCOLORLIST  cdecl cmsAllocNamedColorList(void);
+int                  cdecl cmsReadICCnamedColorList(cmsHTRANSFORM xform, cmsHPROFILE hProfile, icTagSignature sig);
+void                 cdecl cmsFreeNamedColorList(LPcmsNAMEDCOLORLIST List);
+BOOL                 cdecl cmsAppendNamedColor(cmsHTRANSFORM xform, const char* Name, WORD PCS[3], WORD Colorant[MAXCHANNELS]);
+
+
+// I/O
+
+#define MAX_TABLE_TAG       50
+
+// This is the internal struct holding profile details.
+
+typedef struct {
+
+              void*           stream;   // Associated stream. If NULL,
+                                        // tags are supposed to be in
+                                        // memory rather than in a file.
+
+               // Only most important items found in ICC profile
+
+               icProfileClassSignature DeviceClass;
+               icColorSpaceSignature   ColorSpace;
+               icColorSpaceSignature   PCS;
+               icRenderingIntent       RenderingIntent;
+               icUInt32Number          flags;
+               cmsCIEXYZ               Illuminant;
+               
+               // Additions for V4 profiles
+
+               int                     Version;
+               MAT3                    ChromaticAdaptation;
+               cmsCIEXYZ               MediaWhitePoint;
+               cmsCIEXYZ               MediaBlackPoint;
+
+
+               // Dictionary
+
+               icInt32Number   TagCount;
+               icTagSignature  TagNames[MAX_TABLE_TAG];
+               size_t          TagSizes[MAX_TABLE_TAG];
+               size_t          TagOffsets[MAX_TABLE_TAG];
+               LPVOID          TagPtrs[MAX_TABLE_TAG];
+
+
+               char            PhysicalFile[MAX_PATH];
+               
+               BOOL            IsWrite;
+
+               // I/O handlers
+
+               size_t (* Read)(void *buffer, size_t size, size_t count, void *stream);
+               
+               BOOL   (* Seek)(void* stream, size_t offset);
+               BOOL   (* Close)(void* stream);
+               size_t (* Tell)(void* stream);
+               
+               // Writting
+
+               BOOL   (* Write)(void* stream, size_t size, LPVOID Ptr);
+             
+              } LCMSICCPROFILE, FAR* LPLCMSICCPROFILE;
+
+
+// Create an empty template for virtual profiles
+cmsHPROFILE cdecl _cmsCreateProfilePlaceholder(void);
+
+
 // These macros unpack format specifiers into integers
 
 #define T_COLORSPACE(s)       (((s)>>16)&31)
@@ -1154,6 +1432,9 @@ typedef struct _cmstransform_struct {
                             cmsHPROFILE InputProfile;
                             cmsHPROFILE OutputProfile;
                             cmsHPROFILE PreviewProfile;
+
+                            icColorSpaceSignature EntryColorSpace;
+                            icColorSpaceSignature ExitColorSpace;
                         
                             WMAT3 m1, m2;       // Matrix holding inter PCS operation
                             WVEC3 of1, of2;     // Offset terms
@@ -1190,6 +1471,9 @@ typedef struct _cmstransform_struct {
 
                             int Phase1, Phase2, Phase3;
 
+                            // Named color table
+                            LPcmsNAMEDCOLORLIST NamedColorList;
+                                            
 
                             } _cmsTRANSFORM,FAR *_LPcmsTRANSFORM;
 
@@ -1197,8 +1481,9 @@ typedef struct _cmstransform_struct {
 
 // Packing & Unpacking
 
-_cmsFIXFN cdecl _cmsIdentifyInputFormat(DWORD dwInput);
-_cmsFIXFN cdecl _cmsIdentifyOutputFormat(DWORD dwOutput);
+_cmsFIXFN cdecl _cmsIdentifyInputFormat(_LPcmsTRANSFORM xform,  DWORD dwInput);
+_cmsFIXFN cdecl _cmsIdentifyOutputFormat(_LPcmsTRANSFORM xform, DWORD dwOutput);
+
 
 // Conversion
 
@@ -1208,17 +1493,17 @@ _cmsFIXFN cdecl _cmsIdentifyOutputFormat(DWORD dwOutput);
 #define LabAbs       3
 
 
-
-
 int cdecl cmsChooseCnvrt(int Absolute,
                  int Phase1, LPcmsCIEXYZ BlackPointIn,
                              LPcmsCIEXYZ WhitePointIn,
                              LPcmsCIEXYZ IlluminantIn,
+                             LPMAT3 ChromaticAdaptationMatrixIn,
 
                  int Phase2, LPcmsCIEXYZ BlackPointOut,
                              LPcmsCIEXYZ WhitePointOut,
                              LPcmsCIEXYZ IlluminantOut,
-
+                             LPMAT3 ChromaticAdaptationMatrixOut,
+                 int DoBPC,
                  _cmsADJFN *fn1,
                  LPWMAT3 wm, LPWVEC3 wof);
 
@@ -1231,35 +1516,39 @@ BOOL cdecl   _cmsEndPointsBySpace(icColorSpaceSignature Space,
 
 WORD * cdecl _cmsWhiteBySpace(icColorSpaceSignature Space);
 
+
 WORD cdecl Clamp_XYZ(int in);
 WORD cdecl Clamp_RGB(int in);
 
 WORD cdecl Clamp_L(Fixed32 in);
 WORD cdecl Clamp_ab(Fixed32 in);
 
+// Detection of black point
+int cdecl cmsDetectBlackPoint(LPcmsCIEXYZ BlackPoint, cmsHPROFILE hProfile, int Intent);
+
+// choose reasonable resolution
+int cdecl _cmsReasonableGridpointsByColorspace(icColorSpaceSignature Colorspace, DWORD dwFlags);
 
 // Precalculate device link
-BOOL cdecl _cmsPrecalculateDeviceLink(_LPcmsTRANSFORM p, int PrecissionLevel);
+LPLUT cdecl _cmsPrecalculateDeviceLink(cmsHTRANSFORM h, DWORD dwFlags);
 
 // Compute gamut boundary
-LPLUT cdecl _cmsComputeGamutLUT(cmsHPROFILE hProfile);
+LPLUT cdecl _cmsComputeGamutLUT(cmsHPROFILE hProfile, int Intent);
 
 // Compute softproof
 LPLUT _cmsComputeSoftProofLUT(cmsHPROFILE hProfile, int nIntent);
 
 // Find a suitable prelinearization tables, matching the given transform
-void cdecl _cmsComputePrelinearizationTablesFromXFORM(cmsHTRANSFORM h, LPLUT Grid);
+void cdecl _cmsComputePrelinearizationTablesFromXFORM(cmsHTRANSFORM h[], int nTransforms, LPLUT Grid);
 
 // These are two VITAL macros, from converting between 8 and 16 bit
 // representation. 
 
-#define RGB_8_TO_16(rgb) (WORD) ((((WORD) (rgb)) << 8)|(rgb)) // * 257
+#define RGB_8_TO_16(rgb) (WORD) ((((WORD) (rgb)) << 8)|(rgb)) 
+#define RGB_16_TO_8(rgb) (BYTE) ((((rgb) * 65281 + 8388608) >> 24) & 0xFF)
 
-#ifdef USE_257_FOR_16_TO_8_CONVERSION
-#   define RGB_16_TO_8(rgb) (BYTE) (((WORD) (rgb) + 128) / 257)
-#else
-#   define RGB_16_TO_8(rgb) (BYTE) (((WORD) (rgb)) >> 8)
-#endif
+
+#endif  // LCMS_APIONLY
 
 
 #define __cms_H
@@ -1268,7 +1557,5 @@ void cdecl _cmsComputePrelinearizationTablesFromXFORM(cmsHTRANSFORM h, LPLUT Gri
 }
 #endif
 
-
-
 #endif
 
