@@ -28,7 +28,7 @@
 // License along with this library; if not, write to the Free Software
 // Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
-// Version 1.09c
+// Version 1.10c
 
 // This program does apply profiles to (some) JPEG files
 
@@ -62,7 +62,11 @@ static BOOL BlackWhiteCompensation = FALSE;
 static BOOL IgnoreEmbedded         = FALSE;
 static BOOL GamutCheck             = FALSE;
 static int Intent                  = INTENT_PERCEPTUAL;
+static int ProofingIntent          = INTENT_PERCEPTUAL;
 static int PrecalcMode             = 1;
+static BOOL lIsDeviceLink          = FALSE;
+static int jpegQuality = 75;
+
 static char *cInpProf  = NULL;
 static char *cOutProf  = NULL;
 static char *cProofing = NULL;
@@ -72,10 +76,12 @@ static struct jpeg_decompress_struct Decompressor;
 static struct jpeg_compress_struct   Compressor;
 static BOOL   lIsITUFax = FALSE;
 
+
 static struct my_error_mgr {
 
     struct  jpeg_error_mgr pub;  // "public" fields
     LPVOID  Cargo;               // "private" fields
+
 } ErrorHandler;
 
 static
@@ -136,14 +142,14 @@ my_output_message (j_common_ptr cinfo)
   char buffer[JMSG_LENGTH_MAX];
 
   (*cinfo->err->format_message) (cinfo, buffer);
-  ConsoleWarningHandler(PROGNAME, buffer, NULL);
+  FatalError("%s %s", PROGNAME, buffer);  
 }
 
 
 METHODDEF(void)
 my_error_exit (j_common_ptr cinfo)
 {
-  struct my_error_mgr *myerr = (struct my_error_mgr *) cinfo->err;
+  // struct my_error_mgr *myerr = (struct my_error_mgr *) cinfo->err;
   char buffer[JMSG_LENGTH_MAX];
   
   (*cinfo->err->format_message) (cinfo, buffer);
@@ -228,8 +234,10 @@ BOOL OpenOutput(const char* FileName)
         ErrorHandler.pub.output_message  = my_output_message;
 
         Compressor.input_components = Compressor.num_components = 4;
+        
         jpeg_create_compress(&Compressor);
-        jpeg_stdio_dest(&Compressor, OutFile);      
+        jpeg_stdio_dest(&Compressor, OutFile);  
+        
         return TRUE;
 }
 
@@ -411,23 +419,25 @@ static
 void WriteOutputFields(int OutputColorSpace)
 {
     int n, m, k;
-
+    
     switch (OutputColorSpace) {
-
+        
     case PT_GRAY: n = m = JCS_GRAYSCALE; k = 1; break;
     case PT_RGB:  n = JCS_RGB; m = JCS_YCbCr; k = 3; break;       // red/green/blue
-    case PT_YCbCr: n = m = JCS_YCbCr; k = 3; break;    // Y/Cb/Cr (also known as YUV)
-    case PT_CMYK: n = m = JCS_CMYK; k = 4; break;      // C/M/Y/K              
-    case PT_Lab:  n = m = JCS_YCbCr; k = 3; break;      // Fake to don't touch
+    case PT_YCbCr: n = m = JCS_YCbCr; k = 3; break;               // Y/Cb/Cr (also known as YUV)
+    case PT_CMYK: n = m = JCS_CMYK;  k = 4; break;                // C/M/Y/K              
+    case PT_Lab:  n = m = JCS_YCbCr; k = 3; break;                // Fake to don't touch
     default:
-                   FatalError("Unsupported output color space");
-				   return;
+        FatalError("Unsupported output color space");
+        return;
     }
-
+    
     Compressor.in_color_space = n;
     Compressor.jpeg_color_space = m;
     Compressor.input_components = Compressor.num_components = k;
     jpeg_set_defaults(&Compressor);
+    jpeg_set_colorspace(&Compressor, m);
+    jpeg_set_quality(&Compressor, jpegQuality,1);    
 }
 
 static
@@ -497,6 +507,7 @@ int TransformImage(char *cDefInpProf, char *cOutProf)
            
        case 0: dwFlags |= cmsFLAGS_NOTPRECALC; break;
        case 2: dwFlags |= cmsFLAGS_HIGHRESPRECALC; break;
+       case 3: dwFlags |= cmsFLAGS_LOWRESPRECALC; break;
        default:;
        }
         
@@ -505,13 +516,21 @@ int TransformImage(char *cDefInpProf, char *cOutProf)
             dwFlags |= cmsFLAGS_GAMUTCHECK;
         
 
-       if (!IgnoreEmbedded && read_icc_profile(&Decompressor, &EmbedBuffer, &EmbedLen))
-       {
+        if (lIsDeviceLink) {
+
+            hIn = cmsOpenProfileFromFile(cDefInpProf, "r");
+            hOut = NULL;
+            hProof = NULL;
+       }
+        else {
+
+        if (!IgnoreEmbedded && read_icc_profile(&Decompressor, &EmbedBuffer, &EmbedLen))
+        {
               hIn = cmsOpenProfileFromMem(EmbedBuffer, EmbedLen);
               if (Verbose) fprintf(stdout, " (embedded profile found)");
-       }
-       else
-       {
+        }
+        else
+        {
               if (cDefInpProf == NULL)
                   hIn = cmsCreate_sRGBProfile();
               else
@@ -529,6 +548,7 @@ int TransformImage(char *cDefInpProf, char *cOutProf)
 
            hProof = cmsOpenProfileFromFile(cProofing, "r");
        }
+        }
 
        // Take input color space
 
@@ -536,28 +556,38 @@ int TransformImage(char *cDefInpProf, char *cOutProf)
 
        // Assure both, input profile and input JPEG are on same colorspace
 
+       
        if (cmsGetColorSpace(hIn) != _cmsICCcolorSpace(T_COLORSPACE(wInput)))
               FatalError("Input profile is not operating in proper color space");
+       
 
        // Output colorspace is given by output profile
 
-       OutputColorSpace = GetProfileColorSpace(hOut);
+        if (lIsDeviceLink) {
+            OutputColorSpace = T_COLORSPACE(wInput);
+        }
+        else {
+            OutputColorSpace = GetProfileColorSpace(hOut);
+        }
+
        wOutput      = ComputeOutputFormatDescriptor(wInput, OutputColorSpace);
        
        jpeg_copy_critical_parameters(&Decompressor, &Compressor);
        
        WriteOutputFields(OutputColorSpace);               
-
+       
        xform = cmsCreateProofingTransform(hIn, wInput, 
                                           hOut, wOutput, 
                                           hProof, Intent, 
-                                          INTENT_ABSOLUTE_COLORIMETRIC, dwFlags);
+                                          ProofingIntent, dwFlags);
 
        // Handle tile by tile or strip by strip
 
        DoTransform(xform);
-       jcopy_markers_execute(&Decompressor, &Compressor);
 
+       
+       jcopy_markers_execute(&Decompressor, &Compressor);
+       
        cmsDeleteTransform(xform);
        cmsCloseProfile(hIn);
        cmsCloseProfile(hOut);
@@ -572,7 +602,7 @@ int TransformImage(char *cDefInpProf, char *cOutProf)
 static
 void Help(int level)
 {
-     fprintf(stderr, "little cms ICC profile applier for JPEG - v1.0\n\n");
+     fprintf(stderr, "little cms ICC profile applier for JPEG - v1.4\n\n");
 
      switch(level) {
 
@@ -586,16 +616,22 @@ void Help(int level)
      fprintf(stderr, "%ci<profile> - Input profile (defaults to sRGB)\n", SW);
      fprintf(stderr, "%co<profile> - Output profile (defaults to sRGB)\n", SW);   
      fprintf(stderr, "%ct<0,1,2,3> - Intent (0=Perceptual, 1=Colorimetric, 2=Saturation, 3=Absolute)\n", SW);    
+     fprintf(stderr, "%cm<0,1,2,3> - SoftProof intent\n", SW);
      fprintf(stderr, "\n");
 
      fprintf(stderr, "%cb - Black/White compensation\n", SW);
      fprintf(stderr, "%cn - Ignore embedded profile\n", SW);
-     fprintf(stderr, "%cc<0,1,2> - Precalculates transform (0=off, 1=normal, 2=Hi-res) [defaults to 1]\n", SW);     
+
+
+     fprintf(stderr, "%cc<0,1,2,3> - Precalculates transform (0=Off, 1=Normal, 2=Hi-res, 3=LoRes) [defaults to 1]\n", SW);     
      fprintf(stderr, "\n");
 
      fprintf(stderr, "%cp<profile> - Soft proof profile\n", SW);
      fprintf(stderr, "%cg - Marks out-of-gamut colors on softproof\n", SW);
-     
+
+     fprintf(stderr, "\n");
+     fprintf(stderr, "%cq<0..100> - Output JPEG quality\n", SW);
+
      fprintf(stderr, "\n");
      fprintf(stderr, "%ch<0,1,2> - More help\n", SW);
      break;
@@ -638,7 +674,7 @@ void HandleSwitches(int argc, char *argv[])
 {
        int s;
       
-       while ((s=xgetopt(argc,argv,"bBnNvVGgh:H:i:I:o:O:P:p:t:T:c:C:")) != EOF) {
+       while ((s=xgetopt(argc,argv,"bBnNvVGgh:H:i:I:o:O:P:p:t:T:c:C:Q:q:M:m:L:l:")) != EOF) {
 
        switch (s)
        {
@@ -655,14 +691,28 @@ void HandleSwitches(int argc, char *argv[])
 
        case 'i':
        case 'I':
+
+            if (lIsDeviceLink)
+                   FatalError("Device-link already specified"); 
+
             cInpProf = xoptarg;
             break;
 
        case 'o':
        case 'O':
-           cOutProf = xoptarg;
+
+             if (lIsDeviceLink)
+                   FatalError("Device-link already specified"); 
+
+            cOutProf = xoptarg;
             break;
 
+       case 'l':
+       case 'L': 
+                cInpProf = xoptarg;
+                lIsDeviceLink = TRUE;
+                break;
+
        case 'p':
        case 'P':
            cProofing = xoptarg;
@@ -701,6 +751,21 @@ void HandleSwitches(int argc, char *argv[])
             }
             break;
 
+        case 'q':
+        case 'Q':
+            jpegQuality = atoi(xoptarg);
+            if (jpegQuality > 100) jpegQuality = 100;
+            if (jpegQuality < 0)   jpegQuality = 0;
+            break;
+
+       case 'm':
+       case 'M':
+            ProofingIntent = atoi(xoptarg);
+            if (ProofingIntent > 3) ProofingIntent = 3;
+            if (ProofingIntent < 0) ProofingIntent = 0;
+            break;
+
+
   default:
 
        FatalError("Unknown option - run without args to see valid ones");
@@ -727,13 +792,22 @@ int main(int argc, char* argv[])
               }
 
      
-      if (Verbose) 
-        fprintf(stdout, "%s(%s) -> %s(%s) [%s]", argv[xoptind],
+
+      if (Verbose) {
+
+          if (lIsDeviceLink)
+            fprintf(stdout, "%s(device link) -> %s [%s]", 
+                                                argv[xoptind], 
+                                                argv[xoptind+1], 
+                                                Intents[Intent]);
+
+        else
+            fprintf(stdout, "%s(%s) -> %s(%s) [%s]", argv[xoptind],
                                                 (cInpProf == NULL ? "sRGB": cInpProf), 
                                                 argv[xoptind+1],
                                                 (cOutProf == NULL ? "sRGB" : cOutProf), 
                                                 Intents[Intent]);
-
+      }
 
       OpenInput(argv[xoptind]);
       OpenOutput(argv[xoptind+1]);
