@@ -60,32 +60,23 @@ always.
 
 */
 
-/*
-       Conversion between Absolute/relativ Lab, XYZ
 
 
-#define XYZRel       0
-#define LabRel       1
-#define XYZAbs       2
-#define LabAbs       3
-
-
-
-typedef void (* _cmsADJFN)(WORD In[], WORD Out[], LPWVEC3 a, LPWVEC3 b);
-*/
-
 
 int cdecl cmsChooseCnvrt(int Absolute,
                  int Phase1, LPcmsCIEXYZ BlackPointIn,
                              LPcmsCIEXYZ WhitePointIn,
                              LPcmsCIEXYZ IlluminantIn,
+                             LPMAT3 ChromaticAdaptationMatrixIn,
 
                  int Phase2, LPcmsCIEXYZ BlackPointOut,
                              LPcmsCIEXYZ WhitePointOut,
                              LPcmsCIEXYZ IlluminantOut,
+                             LPMAT3 ChromaticAdaptationMatrixOut,
 
+                int DoBlackPointCompensation,
                  _cmsADJFN *fn1,
-                 LPWVEC3 wa, LPWVEC3 wb);
+                 LPWMAT3 wm, LPWVEC3 wof);
 
 
 // -------------------------------------------------------------------------
@@ -94,66 +85,58 @@ int cdecl cmsChooseCnvrt(int Absolute,
 
 LCMSAPI LPcmsCIEXYZ LCMSEXPORT cmsD50_XYZ(void)
 {
-	static cmsCIEXYZ D50XYZ = {D50X, D50Y, D50Z};
+    static cmsCIEXYZ D50XYZ = {D50X, D50Y, D50Z};
 
-	return &D50XYZ;
+    return &D50XYZ;
 }
 
 LCMSAPI LPcmsCIExyY LCMSEXPORT cmsD50_xyY(void)
 {
-	static cmsCIExyY D50xyY = {D50x, D50y, D50Y};
+    static cmsCIExyY D50xyY;
+    cmsXYZ2xyY(&D50xyY, cmsD50_XYZ());
 
-	return &D50xyY;
+    return &D50xyY;
 }
 
 
 // ---------------- From LUT to LUT --------------------------
 
 
-// Calculate a, b for Relativ -> Absolute
-//
-//     Out = BP+(WP-BP)/Ill*In
-//
-//      b = MediaBlack
-//     a = (MediaWhite - MediaBlack) / Illuminant
-//
+// Calculate m, offset Relativ -> Absolute undoing any chromatic 
+// adaptation done by the profile. 
+
+#ifdef _MSC_VER
+#pragma warning(disable : 4100 4505)
+#endif
 
 static
 void Rel2AbsCoefs(LPcmsCIEXYZ BlackPoint,
                   LPcmsCIEXYZ WhitePoint,
                   LPcmsCIEXYZ Illuminant,
-                  LPVEC3 a, LPVEC3 b)
+                  LPMAT3 ChromaticAdaptationMatrix,
+                  LPMAT3 m, LPVEC3 of)
 {
-
-       VEC3init(b, -BlackPoint->X, -BlackPoint->Y, -BlackPoint->Z);
-
-       VEC3init(a, (WhitePoint->X - BlackPoint->X) / Illuminant -> X,
-                   (WhitePoint->Y - BlackPoint->Y) / Illuminant -> Y,
-                   (WhitePoint->Z - BlackPoint->Z) / Illuminant -> Z);
+     MAT3 Chrm;
+            
+       Chrm = *ChromaticAdaptationMatrix;
+       MAT3inverse(&Chrm, m);
+       VEC3init(of, 0, 0, 0);     
+      
 }
 
-// Calculate a, b for Absolute -> Relativ
-//
-//  Out = (In - BP) * Ill / (WP - BP) == Ill/(WP-BP) * In - BP*Ill/(WP-BP)
-//
+// Calculate a, b for Absolute -> Relativ undoing any chromatic adaptation
+// done by the profile
 
 static
 void Abs2RelCoefs(LPcmsCIEXYZ BlackPoint,
                   LPcmsCIEXYZ WhitePoint,
                   LPcmsCIEXYZ Illuminant,
-                  LPVEC3 a, LPVEC3 b)
+                  LPMAT3 ChromaticAdaptationMatrix,
+                  LPMAT3 m, LPVEC3 of)
 {
-       double tx, ty, tz;
-
-       tx = Illuminant -> X / (WhitePoint->X - BlackPoint->X);
-       ty = Illuminant -> Y / (WhitePoint->Y - BlackPoint->Y);
-       tz = Illuminant -> Z / (WhitePoint->Z - BlackPoint->Z);
-
-       VEC3init(a, tx, ty, tz);
-
-       VEC3init(b, -tx * BlackPoint->X,
-                   -ty * BlackPoint->Y,
-                   -tz * BlackPoint->Z);
+      
+       *m = *ChromaticAdaptationMatrix;
+        VEC3init(of, 0, 0, 0);       
 }
 
 // join scalings to obtain:
@@ -163,132 +146,152 @@ static
 void Rel2RelStepAbsCoefs(LPcmsCIEXYZ BlackPointIn,
                       LPcmsCIEXYZ WhitePointIn,
                       LPcmsCIEXYZ IlluminantIn,
+                      LPMAT3 ChromaticAdaptationMatrixIn,
+
                       LPcmsCIEXYZ BlackPointOut,
                       LPcmsCIEXYZ WhitePointOut,
                       LPcmsCIEXYZ IlluminantOut,
-                      LPVEC3 a, LPVEC3 b)
+                      LPMAT3 ChromaticAdaptationMatrixOut,
+
+                      LPMAT3 m, LPVEC3 of)
 {
-       VEC3 ain, bin, aout, bout;
-
+       MAT3 min, mout;
+       VEC3 ofin, ofout;
 
+       
        Rel2AbsCoefs(BlackPointIn,
                     WhitePointIn,
-                    IlluminantIn, &ain, &bin);
+                    IlluminantIn, 
+                    ChromaticAdaptationMatrixIn,
+                    &min, &ofin);
 
        Abs2RelCoefs(BlackPointOut,
                     WhitePointOut,
-                    IlluminantOut, &aout, &bout);
+                    IlluminantOut, 
+                    ChromaticAdaptationMatrixOut,
+                    &mout, &ofout);
 
 
-       VEC3init(a, aout.n[VX]*ain.n[VX],
-                   aout.n[VY]*ain.n[VY],
-                   aout.n[VZ]*ain.n[VZ]);
-
-       VEC3init(b, (aout.n[VX]*bin.n[VX] + bout.n[VX]),
-                   (aout.n[VY]*bin.n[VY] + bout.n[VY]),
-                   (aout.n[VZ]*bin.n[VZ] + bout.n[VZ]));
-
+       // Chromatic adaptation matrix should be conmutative
+       // min * mout = mout * min
+           
+       MAT3per(m,  &min, &mout);
+       VEC3init(of, 0.0, 0.0, 0.0);
+              
 }
 
 
-
-// Calculate scaling between illuminants
+// The (in)famous black point compensation. Right now implemented as
+// a linear scaling in XYZ
 
 static
-void BetweenIlluminants(LPcmsCIEXYZ IlluminantFrom, LPcmsCIEXYZ IlluminantTo, LPVEC3 a)
+void ComputeBlackPointCompensationFactors(LPcmsCIEXYZ BlackPointIn,
+                      LPcmsCIEXYZ WhitePointIn,
+                      LPcmsCIEXYZ IlluminantIn,
+                      LPcmsCIEXYZ BlackPointOut,
+                      LPcmsCIEXYZ WhitePointOut,
+                      LPcmsCIEXYZ IlluminantOut,
+                      LPMAT3 m, LPVEC3 of)
 {
+    
+
+   cmsCIEXYZ RelativeBlackPointIn, RelativeBlackPointOut;
+   double ax, ay, az, bx, by, bz, tx, ty, tz;
+   
+   // At first, convert both black points to relative.
+
+   cmsAdaptToIlluminant(&RelativeBlackPointIn,  WhitePointIn, IlluminantIn, BlackPointIn);
+   cmsAdaptToIlluminant(&RelativeBlackPointOut, WhitePointOut, IlluminantOut, BlackPointOut);
+   
+   // Now we need to compute a matrix plus an offset m and of such of
+   // [m]*bpin + off = bpout
+   // [m]*D50  + off = D50
+   //
+   // This is a linear scaling in the form ax+b, where
+   // a = (bpout - D50) / (bpin - D50)
+   // b = - D50* (bpout - bpin) / (bpin - D50)
+
+
+   tx = RelativeBlackPointIn.X - IlluminantIn ->X;
+   ty = RelativeBlackPointIn.Y - IlluminantIn ->Y;
+   tz = RelativeBlackPointIn.Z - IlluminantIn ->Z;
+
+   ax = (RelativeBlackPointOut.X - IlluminantOut ->X) / tx;
+   ay = (RelativeBlackPointOut.Y - IlluminantOut ->Y) / ty;
+   az = (RelativeBlackPointOut.Z - IlluminantOut ->Z) / tz;
+
+   bx = - IlluminantOut -> X * (RelativeBlackPointOut.X - RelativeBlackPointIn.X) / tx;
+   by = - IlluminantOut -> Y * (RelativeBlackPointOut.Y - RelativeBlackPointIn.Y) / ty;
+   bz = - IlluminantOut -> Z * (RelativeBlackPointOut.Z - RelativeBlackPointIn.Z) / tz;
+
+
+   MAT3identity(m);
+
+   m->v[VX].n[0] = ax;
+   m->v[VY].n[1] = ay;
+   m->v[VZ].n[2] = az;
+
+   VEC3init(of, bx, by, bz);
 
-       VEC3init(a, IlluminantTo->X/IlluminantFrom->X,
-                   IlluminantTo->Y/IlluminantFrom->Y,
-                   IlluminantTo->Z/IlluminantFrom->Z);
 }
 
-// Calculate scaling from illuminant to D50
+// ----------------------------------------- Inter PCS conversions
+
+// XYZ to XYZ linear scaling. Aso used on Black point compensation
 
 static
-void ToD50(LPcmsCIEXYZ Illuminant, LPVEC3 a)
+void XYZ2XYZ(WORD In[], WORD Out[], LPWMAT3 m, LPWVEC3 of)
 {
-       VEC3init(a, D50X/Illuminant->X, D50Y/Illuminant->Y,D50Z/Illuminant->Z);
-}
 
-// Calculate scaling from D50 to illuminant
+    WVEC3 a, r;
 
-static
-void FromD50(LPcmsCIEXYZ Illuminant, LPVEC3 a)
-{
-       VEC3init(a, Illuminant->X/D50X, Illuminant->Y/D50Y,Illuminant->Z/D50Z);
-}
+    a.n[0] = In[0] << 1;
+    a.n[1] = In[1] << 1;
+    a.n[2] = In[2] << 1;
 
+    MAT3evalW(&r, m, &a);
 
-// Simpler identity (not used)
-
-
-#ifdef __BORLANDC__
-#pragma warn -par
-#endif
-
-#ifdef _MSC_VER
-#pragma warning(disable : 4100 4505)
-#endif
-
-static
-void Identity(WORD In[], WORD Out[], LPWVEC3 a, LPWVEC3 b)
-{
-#ifdef __MWERKS__
-#pragma unused(a,b)
-#endif
-       Out[0] = In[0];
-       Out[1] = In[1];
-       Out[2] = In[2];
-}
-
-
-// XYZ to XYZ linear scalling
-
-static
-void XYZ2XYZ(WORD In[], WORD Out[], LPWVEC3 a, LPWVEC3 b)
-{
-  Out[0] = Clamp_XYZ((b->n[0] + FixedMul(a->n[0], ((Fixed32) In[0] << 1)) ) >> 1);
-  Out[1] = Clamp_XYZ((b->n[1] + FixedMul(a->n[1], ((Fixed32) In[1] << 1)) ) >> 1);
-  Out[2] = Clamp_XYZ((b->n[2] + FixedMul(a->n[2], ((Fixed32) In[2] << 1)) ) >> 1);
+    Out[0] = Clamp_XYZ((r.n[VX] + of->n[VX]) >> 1);
+    Out[1] = Clamp_XYZ((r.n[VY] + of->n[VY]) >> 1);
+    Out[2] = Clamp_XYZ((r.n[VZ] + of->n[VZ]) >> 1);
 }
 
 
 // XYZ to Lab, scaling first
 
 static
-void XYZ2Lab(WORD In[], WORD Out[], LPWVEC3 a, LPWVEC3 b)
+void XYZ2Lab(WORD In[], WORD Out[], LPWMAT3 m, LPWVEC3 of)
 {
   WORD XYZ[3];
 
-  XYZ2XYZ(In, XYZ, a, b);
+  XYZ2XYZ(In, XYZ, m, of);
   cmsXYZ2LabEncoded(XYZ, Out);
 }
 
 // Lab to XYZ, then scalling
 
 static
-void Lab2XYZ(WORD In[], WORD Out[], LPWVEC3 a, LPWVEC3 b)
+void Lab2XYZ(WORD In[], WORD Out[], LPWMAT3 m, LPWVEC3 of)
 {
        WORD XYZ[3];
 
        cmsLab2XYZEncoded(In, XYZ);
-       XYZ2XYZ(XYZ, Out, a, b);
+       XYZ2XYZ(XYZ, Out, m, of);
 }
 
 // Lab to XYZ, scalling and then, back to Lab
 
 static
-void Lab2XYZ2Lab(WORD In[], WORD Out[], LPWVEC3 a, LPWVEC3 b)
+void Lab2XYZ2Lab(WORD In[], WORD Out[], LPWMAT3 m, LPWVEC3 of)
 {
        WORD XYZ[3], XYZ2[3];
 
        cmsLab2XYZEncoded(In, XYZ);
-       XYZ2XYZ(XYZ, XYZ2, a, b);
+       XYZ2XYZ(XYZ, XYZ2, m, of);
        cmsXYZ2LabEncoded(XYZ2, Out);
 }
 
-
+// ------------------------------------------------------------------
 
 // Dispatcher for XYZ Relative LUT
 
@@ -297,13 +300,16 @@ int FromXYZRelLUT(int Absolute,
                              LPcmsCIEXYZ BlackPointIn,
                              LPcmsCIEXYZ WhitePointIn,
                              LPcmsCIEXYZ IlluminantIn,
+                             LPMAT3 ChromaticAdaptationMatrixIn,
 
                  int Phase2, LPcmsCIEXYZ BlackPointOut,
                              LPcmsCIEXYZ WhitePointOut,
                              LPcmsCIEXYZ IlluminantOut,
+                             LPMAT3 ChromaticAdaptationMatrixOut,
 
+                 int DoBlackPointCompensation,
                  _cmsADJFN *fn1,
-                 LPVEC3 a, LPVEC3 b)
+                 LPMAT3 m, LPVEC3 of)
 
 {
               switch (Phase2) {
@@ -320,30 +326,43 @@ int FromXYZRelLUT(int Absolute,
                                    Rel2RelStepAbsCoefs(BlackPointIn,
                                                   WhitePointIn,
                                                   IlluminantIn,
+                                                  ChromaticAdaptationMatrixIn,
                                                   BlackPointOut,
                                                   WhitePointOut,
                                                   IlluminantOut,
-                                                  a, b);
+                                                  ChromaticAdaptationMatrixOut,
+                                                  m, of);
                                    *fn1 = XYZ2XYZ;
 
                             }
                             else
                             {
-                                   // Relative to relative, only
-                                   // adjust the illuminants
+                                   // XYZ Relative to XYZ relative, no op required                                   
+                                   *fn1 = NULL;
+                                   if (DoBlackPointCompensation) {
 
-                                   BetweenIlluminants(IlluminantIn,
-                                                      IlluminantOut, a);
-                                   VEC3init(b, 0, 0, 0);
-                                   *fn1 = XYZ2XYZ;
+                                      *fn1 = XYZ2XYZ;
+                                      ComputeBlackPointCompensationFactors(BlackPointIn,
+                                                                      WhitePointIn,
+                                                                      IlluminantIn,
+                                                                      BlackPointOut,
+                                                                      WhitePointOut,
+                                                                      IlluminantOut,
+                                                                      m, of);
+
+                                   }
                             }
                             break;
 
-                     // From relative XYZ to absolute XYZ. Always absolute
+                     // From relative XYZ to absolute XYZ. 
 
                      case XYZAbs:
 
-                            Rel2AbsCoefs(BlackPointIn, WhitePointIn, IlluminantIn, a, b);
+                            Rel2AbsCoefs(BlackPointIn, 
+                                         WhitePointIn, 
+                                         IlluminantIn, 
+                                         ChromaticAdaptationMatrixIn,
+                                         m, of);
                             *fn1 = XYZ2XYZ;
                             break;
 
@@ -356,34 +375,39 @@ int FromXYZRelLUT(int Absolute,
                             // to prepare the "to Lab" conversion.
 
                             if (Absolute)
-                            {
-                                VEC3 a1, b1, btw;
+                            {   
 
                                 Rel2RelStepAbsCoefs(BlackPointIn,
                                                     WhitePointIn,
                                                     IlluminantIn,
+                                                    ChromaticAdaptationMatrixIn,
                                                     BlackPointOut,
                                                     WhitePointOut,
                                                     IlluminantOut,
-                                                    &a1, &b1);
-
-                                BetweenIlluminants(IlluminantIn,
-                                                   cmsD50_XYZ(),
-                                                   &btw);
-
-                                VEC3perComp(a, &a1, &btw);
-                                VEC3perComp(b, &b1, &btw);
+                                                    ChromaticAdaptationMatrixOut,
+                                                    m, of);
+                                
                                 *fn1 = XYZ2Lab;
 
                             }
                             else
                             {
-                                   // Only adjust illuminant to D50
+                                   // Just Convert to Lab
 
-                                   BetweenIlluminants(IlluminantIn,
-                                                      cmsD50_XYZ(), a);
-                                   VEC3init(b, 0, 0, 0);
+                                   MAT3identity(m);
+                                   VEC3init(of, 0, 0, 0);
                                    *fn1 = XYZ2Lab;
+
+                                   if (DoBlackPointCompensation) {
+
+                                    ComputeBlackPointCompensationFactors(BlackPointIn,
+                                                                          WhitePointIn,
+                                                                          IlluminantIn,
+                                                                          BlackPointOut,
+                                                                          WhitePointOut,
+                                                                          IlluminantOut,
+                                                                          m, of);
+                                   }
                             }
                             break;
 
@@ -391,22 +415,16 @@ int FromXYZRelLUT(int Absolute,
 
                      case LabAbs:
                                 {
-                                VEC3 a1, b1, btw;
-
                                 Rel2RelStepAbsCoefs(BlackPointIn,
                                                     WhitePointIn,
                                                     IlluminantIn,
+                                                    ChromaticAdaptationMatrixIn,
                                                     BlackPointOut,
                                                     WhitePointOut,
                                                     IlluminantOut,
-                                                    &a1, &b1);
-
-                                BetweenIlluminants(IlluminantOut,
-                                                   cmsD50_XYZ(),
-                                                   &btw);
-
-                                VEC3perComp(a, &a1, &btw);
-                                VEC3perComp(b, &b1, &btw);
+                                                    ChromaticAdaptationMatrixOut,
+                                                    m, of);
+                                
                                 *fn1 = XYZ2Lab;
                                 }
                             break;
@@ -427,16 +445,19 @@ int FromXYZAbsLUT(
                  int Phase2, LPcmsCIEXYZ BlackPointOut,
                              LPcmsCIEXYZ WhitePointOut,
                              LPcmsCIEXYZ IlluminantOut,
+                             LPMAT3 ChromaticAdaptationMatrixOut,
 
                  _cmsADJFN *fn1,
-                 LPVEC3 a, LPVEC3 b)
+                 LPMAT3 m, LPVEC3 of)
 
 {
 
           switch (Phase2) {
 
               case XYZRel:
-                     Abs2RelCoefs(BlackPointOut, WhitePointOut, IlluminantOut, a, b);
+                     Abs2RelCoefs(BlackPointOut, 
+                                  WhitePointOut, IlluminantOut, 
+                                  ChromaticAdaptationMatrixOut, m, of);
                      *fn1 = XYZ2XYZ;
                      break;
 
@@ -445,12 +466,13 @@ int FromXYZAbsLUT(
                      break;
 
               case LabRel:
-                     Abs2RelCoefs(BlackPointOut, WhitePointOut, IlluminantOut, a, b);
+                     Abs2RelCoefs(BlackPointOut, 
+                                  WhitePointOut, IlluminantOut, 
+                                  ChromaticAdaptationMatrixOut, m, of);
                      *fn1 = XYZ2Lab;
                      break;
 
               case LabAbs:
-                     FromD50(IlluminantOut, a);
                      *fn1 = XYZ2Lab;
                      break;
 
@@ -467,13 +489,16 @@ int FromLabRelLUT(int Absolute,
                              LPcmsCIEXYZ BlackPointIn,
                              LPcmsCIEXYZ WhitePointIn,
                              LPcmsCIEXYZ IlluminantIn,
+                             LPMAT3 ChromaticAdaptationMatrixIn,
 
                  int Phase2, LPcmsCIEXYZ BlackPointOut,
                              LPcmsCIEXYZ WhitePointOut,
                              LPcmsCIEXYZ IlluminantOut,
+                             LPMAT3 ChromaticAdaptationMatrixOut,
 
+                int DoBlackPointCompensation,
                  _cmsADJFN *fn1,
-                 LPVEC3 a, LPVEC3 b)
+                 LPMAT3 m, LPVEC3 of)
 {
 
           switch (Phase2) {
@@ -482,31 +507,40 @@ int FromLabRelLUT(int Absolute,
 
               case XYZRel:
 
-                     if (Absolute)
-                     {
+                  if (Absolute) {  // Absolute intent
+
                             // From lab relative, to XYZ absolute, and then,
                             // back to XYZ relative
 
                             Rel2RelStepAbsCoefs(BlackPointIn,
                                            WhitePointIn,
                                            cmsD50_XYZ(),
+                                           ChromaticAdaptationMatrixIn,
                                            BlackPointOut,
                                            WhitePointOut,
                                            IlluminantOut,
-                                           a, b);
+                                           ChromaticAdaptationMatrixOut,
+                                           m, of);
 
                             *fn1 = Lab2XYZ;
 
                      }
                      else
                      {
-                            // From Lab relative, to XYZ absolute. It is
-                            // needed to adjust illuminants, but since input
-                            // is Lab, it is only necessary adjust from D50
-
-                            FromD50(IlluminantOut, a);
-                            VEC3init(b, 0, 0, 0);
+                            // From Lab relative, to XYZ relative.
+                            
                             *fn1 = Lab2XYZ;
+                            if (DoBlackPointCompensation) {
+
+                                 ComputeBlackPointCompensationFactors(BlackPointIn,
+                                                                      WhitePointIn,
+                                                                      IlluminantIn,
+                                                                      BlackPointOut,
+                                                                      WhitePointOut,
+                                                                      IlluminantOut,
+                                                                      m, of);
+
+                            }
                      }
                      break;
 
@@ -514,33 +548,59 @@ int FromLabRelLUT(int Absolute,
               // From Relative Lab to XYZ absolute. First covert to relative XYZ,
               // then to absolute XYZ
 
-              case XYZAbs:
-                     {
-                     Rel2AbsCoefs(BlackPointIn, WhitePointIn, cmsD50_XYZ(), a, b);
+              case XYZAbs: {
+
+                     Rel2AbsCoefs(BlackPointIn, 
+                                  WhitePointIn, 
+                                  cmsD50_XYZ(), 
+                                  ChromaticAdaptationMatrixIn,
+                                  m, of);
+
                      *fn1 = Lab2XYZ;
                      }
                      break;
 
               case LabRel:
-                     if (Absolute)
-                     {
+
+                     if (Absolute) {
+
                      // First pass to XYZ using the input illuminant
                      // * InIlluminant / D50, then to absolute. Then
                      // to relative, but for input
 
-                     Rel2RelStepAbsCoefs(BlackPointIn, WhitePointIn, IlluminantIn,
-                                         BlackPointOut, WhitePointOut, cmsD50_XYZ(),
-                                         a, b);
+                     Rel2RelStepAbsCoefs(BlackPointIn, 
+                                         WhitePointIn, IlluminantIn,
+                                         ChromaticAdaptationMatrixIn,
+                                         BlackPointOut, 
+                                         WhitePointOut, cmsD50_XYZ(),
+                                         ChromaticAdaptationMatrixOut,
+                                         m, of);
                      *fn1 = Lab2XYZ2Lab;
                      }
                      else
-                     {
-                     *fn1 = NULL;
+                     {      // Lab -> Lab relative don't need any adjust unless
+                            // black point compensation
+
+                            *fn1 = NULL;
+                             if (DoBlackPointCompensation) {
+
+                                 *fn1 = Lab2XYZ2Lab;
+                                 ComputeBlackPointCompensationFactors(BlackPointIn,
+                                                                      WhitePointIn,
+                                                                      IlluminantIn,
+                                                                      BlackPointOut,
+                                                                      WhitePointOut,
+                                                                      IlluminantOut,
+                                                                      m, of);
+
+                                
+                            }
                      }
                      break;
 
               case LabAbs:
-                     Rel2AbsCoefs(BlackPointIn, WhitePointIn, cmsD50_XYZ(), a, b);
+                     Rel2AbsCoefs(BlackPointIn, WhitePointIn, cmsD50_XYZ(), 
+                         ChromaticAdaptationMatrixIn, m, of);
                      *fn1 = Lab2XYZ2Lab;
                      break;
 
@@ -557,34 +617,36 @@ static
 int FromLabAbsLUT(           LPcmsCIEXYZ BlackPointIn,
                              LPcmsCIEXYZ WhitePointIn,
                              LPcmsCIEXYZ IlluminantIn,
+                             LPMAT3 ChromaticAdaptationMatrixIn,
 
                  int Phase2, LPcmsCIEXYZ BlackPointOut,
                              LPcmsCIEXYZ WhitePointOut,
                              LPcmsCIEXYZ IlluminantOut,
+                             LPMAT3 ChromaticAdaptationMatrixOut,
 
                  _cmsADJFN *fn1,
-                 LPVEC3 a, LPVEC3 b)
+                 LPMAT3 m, LPVEC3 of)
 {
 
 
           switch (Phase2) {
 
-              // From Lab absolute to XYZ Relative, convert first to
-              // Absolute XYZ, then to relative
-
-              case XYZRel:
-                     Abs2RelCoefs(BlackPointIn, WhitePointIn, IlluminantIn, a, b);
-                     ToD50(IlluminantOut, a);
+              
+              case XYZRel:                     
                      *fn1 = Lab2XYZ;
                      break;
 
               case XYZAbs:
-                     Abs2RelCoefs(BlackPointOut, WhitePointOut, IlluminantOut, a, b);
+                     Abs2RelCoefs(BlackPointOut, 
+                                  WhitePointOut, IlluminantOut, 
+                                  ChromaticAdaptationMatrixOut, m, of);
                      *fn1 = Lab2XYZ;
                      break;
 
               case LabRel:
-                     Abs2RelCoefs(BlackPointOut, WhitePointOut, IlluminantOut, a, b);
+                     Abs2RelCoefs(BlackPointOut, 
+                                  WhitePointOut, IlluminantOut, 
+                                  ChromaticAdaptationMatrixOut, m, of);
                      *fn1 = Lab2XYZ2Lab;
                      break;
 
@@ -594,7 +656,7 @@ int FromLabAbsLUT(           LPcmsCIEXYZ BlackPointIn,
 
               default: return FALSE;
               }
-
+        
         return TRUE;
 }
 
@@ -611,24 +673,28 @@ int FromLabAbsLUT(           LPcmsCIEXYZ BlackPointIn,
 // Main dispatcher
 
 int cmsChooseCnvrt(int Absolute,
-                 int Phase1, LPcmsCIEXYZ BlackPointIn,
-                             LPcmsCIEXYZ WhitePointIn,
-                             LPcmsCIEXYZ IlluminantIn,
+                  int Phase1, LPcmsCIEXYZ BlackPointIn,
+                              LPcmsCIEXYZ WhitePointIn,
+                              LPcmsCIEXYZ IlluminantIn,
+                              LPMAT3 ChromaticAdaptationMatrixIn,
 
-                 int Phase2, LPcmsCIEXYZ BlackPointOut,
-                             LPcmsCIEXYZ WhitePointOut,
-                             LPcmsCIEXYZ IlluminantOut,
+                  int Phase2, LPcmsCIEXYZ BlackPointOut,
+                              LPcmsCIEXYZ WhitePointOut,
+                              LPcmsCIEXYZ IlluminantOut,
+                              LPMAT3 ChromaticAdaptationMatrixOut,
 
-                 _cmsADJFN *fn1,
-                 LPWVEC3 wa, LPWVEC3 wb)
+                  int DoBlackPointCompensation,
+                  _cmsADJFN *fn1,
+                  LPWMAT3 wm, LPWVEC3 wof)
 {
 
        int rc;
-       VEC3 a, b;
+       MAT3 m;
+       VEC3 of;
 
 
-       VEC3init(&a, 1, 1, 1);
-       VEC3init(&b, 0, 0, 0);
+       MAT3identity(&m);
+       VEC3init(&of, 0, 0, 0);
 
        switch (Phase1) {
 
@@ -638,21 +704,27 @@ int cmsChooseCnvrt(int Absolute,
                                           BlackPointIn,
                                           WhitePointIn,
                                           IlluminantIn,
+                                          ChromaticAdaptationMatrixIn,
                                           Phase2,
                                           BlackPointOut,
                                           WhitePointOut,
                                           IlluminantOut,
-                                          fn1, &a, &b);
+                                          ChromaticAdaptationMatrixOut,
+                                          DoBlackPointCompensation,
+                                          fn1, &m, &of);
                      break;
 
-       // Input LUT is giving XYZ Absolute values. Always absolute
+       
+       // Input LUT is giving XYZ Absolute values. 
 
        case XYZAbs:  rc = FromXYZAbsLUT(Phase2,
                                           BlackPointOut,
                                           WhitePointOut,
                                           IlluminantOut,
-                                          fn1, &a, &b);
+                                          ChromaticAdaptationMatrixOut,
+                                          fn1, &m, &of);
                      break;
+       
 
        // Input LUT is giving Lab relative values
 
@@ -660,24 +732,31 @@ int cmsChooseCnvrt(int Absolute,
                                           BlackPointIn,
                                           WhitePointIn,
                                           IlluminantIn,
+                                          ChromaticAdaptationMatrixIn,
                                           Phase2,
                                           BlackPointOut,
                                           WhitePointOut,
                                           IlluminantOut,
-                                          fn1, &a, &b);
+                                          ChromaticAdaptationMatrixOut,
+                                          DoBlackPointCompensation,
+                                          fn1, &m, &of);
                      break;
 
-       // Input LUT is giving absolute Lab values, Always absolute
-
+       
+       // Input LUT is giving absolute Lab values.
+        
        case LabAbs:  rc = FromLabAbsLUT(BlackPointIn,
                                           WhitePointIn,
                                           IlluminantIn,
+                                          ChromaticAdaptationMatrixIn,
                                           Phase2,
                                           BlackPointOut,
                                           WhitePointOut,
                                           IlluminantOut,
-                                          fn1, &a, &b);
+                                          ChromaticAdaptationMatrixOut,
+                                          fn1, &m, &of);
                      break;
+        
 
        // Unrecognized combination
 
@@ -686,8 +765,8 @@ int cmsChooseCnvrt(int Absolute,
 
        }
 
-       VEC3toFix(wa, &a);
-       VEC3toFix(wb, &b);
+       MAT3toFix(wm, &m);
+       VEC3toFix(wof, &of);
 
        return rc;
 }
