@@ -57,22 +57,21 @@
 
 // -------------------------------------------------------- Unpacking routines.
 
+
 static
-LPBYTE Unroll6Bytes(register _LPcmsTRANSFORM info, register WORD wIn[], register LPBYTE accum)
+LPBYTE UnrollAnyBytes(register _LPcmsTRANSFORM info, register WORD wIn[], register LPBYTE accum)
 {
+       int nChan = T_CHANNELS(info -> InputFormat);
+       register int i;
 
-       wIn[0] = RGB_8_TO_16(*accum); accum++; 
-       wIn[1] = RGB_8_TO_16(*accum); accum++; 
-       wIn[2] = RGB_8_TO_16(*accum); accum++; 
-       wIn[3] = RGB_8_TO_16(*accum); accum++; 
-       wIn[4] = RGB_8_TO_16(*accum); accum++; 
-       wIn[5] = RGB_8_TO_16(*accum); accum++; 
+       for (i=0; i < nChan; i++) {
 
+              wIn[i] = RGB_8_TO_16(*accum); accum++;              
+       }
 
        return accum;
 }
 
-
 static
 LPBYTE Unroll4Bytes(register _LPcmsTRANSFORM info, register WORD wIn[], register LPBYTE accum)
 {
@@ -127,16 +126,17 @@ LPBYTE Unroll4BytesSwapSwapFirst(register _LPcmsTRANSFORM info, register WORD wI
 
 
 static
-LPBYTE Unroll6Words(register _LPcmsTRANSFORM info, register WORD wIn[], register LPBYTE accum)
+LPBYTE UnrollAnyWords(register _LPcmsTRANSFORM info, register WORD wIn[], register LPBYTE accum)
 {
-       wIn[0] = *(LPWORD) accum; accum+= 2; 
-       wIn[1] = *(LPWORD) accum; accum+= 2; 
-       wIn[2] = *(LPWORD) accum; accum+= 2; 
-       wIn[3] = *(LPWORD) accum; accum+= 2; 
-       wIn[4] = *(LPWORD) accum; accum+= 2; 
-       wIn[5] = *(LPWORD) accum; accum+= 2; 
+     int nChan = T_CHANNELS(info -> InputFormat);
+     register int i;
+     
+     for (i=0; i < nChan; i++) {
 
-       return accum;
+              wIn[i] = *(LPWORD) accum; accum += 2;              
+     }
+
+     return accum;       
 }
 
 
@@ -400,6 +400,8 @@ LPBYTE UnrollPlanarBytes(register _LPcmsTRANSFORM info, register WORD wIn[], reg
        return (Init + 1);
 }
 
+
+
 static
 LPBYTE UnrollPlanarWords(register _LPcmsTRANSFORM info, register WORD wIn[], register LPBYTE accum)
 {
@@ -416,6 +418,8 @@ LPBYTE UnrollPlanarWords(register _LPcmsTRANSFORM info, register WORD wIn[], reg
        return (Init + 2);
 }
 
+
+
 static
 LPBYTE UnrollPlanarWordsBigEndian(register _LPcmsTRANSFORM info, register WORD wIn[], register LPBYTE accum)
 {
@@ -433,6 +437,72 @@ LPBYTE UnrollPlanarWordsBigEndian(register _LPcmsTRANSFORM info, register WORD w
 }
 
 
+// floating point
+static
+LPBYTE UnrollLabDouble(register _LPcmsTRANSFORM info, register WORD wIn[], register LPBYTE accum)
+{       
+        cmsFloat2LabEncoded(wIn, (LPcmsCIELab) accum);
+        accum += sizeof(cmsCIELab);
+
+        return accum;
+}
+
+static
+LPBYTE UnrollXYZDouble(register _LPcmsTRANSFORM info, register WORD wIn[], register LPBYTE accum)
+{       
+        cmsFloat2XYZEncoded(wIn, (LPcmsCIEXYZ) accum);
+        accum += sizeof(cmsCIEXYZ);
+
+        return accum;
+}
+
+// Inks does come in percentage
+static
+LPBYTE UnrollInkDouble(register _LPcmsTRANSFORM info, register WORD wIn[], register LPBYTE accum)
+{
+    double* Inks = (double*) accum;
+    int nChan = T_CHANNELS(info -> InputFormat);
+    int i;
+
+    for (i=0; i <  nChan; i++) {
+
+        double v = floor(Inks[i] * 655.35 + 0.5);
+
+        if (v > 65535.0) v = 65535.0;
+        if (v < 0) v = 0;
+
+        wIn[i] = (WORD) v;
+    }
+
+    return accum + (nChan + T_EXTRA(info ->InputFormat)) * sizeof(double);
+}
+
+
+// Remaining cases are between 0..1.0
+static
+LPBYTE UnrollDouble(register _LPcmsTRANSFORM info, register WORD wIn[], register LPBYTE accum)
+{
+    double* Inks = (double*) accum;
+    int nChan = T_CHANNELS(info -> InputFormat);
+    int i;
+
+    for (i=0; i <  nChan; i++) {
+
+        double v = floor(Inks[i] * 65535.0 + 0.5);
+
+        if (v > 65535.0) 
+            v = 65535.0;
+        if (v < 0) 
+            v = 0;
+
+        wIn[i] = (WORD) v;
+    }
+
+    return accum + (nChan + T_EXTRA(info ->InputFormat)) * sizeof(double);
+}
+
+
+
 // ----------------------------------------------------------- Packing routines
 
 
@@ -1060,24 +1130,134 @@ LPBYTE Pack1WordAndSkip1BigEndian(register _LPcmsTRANSFORM Info, register WORD w
 }
 
 
+// Unencoded Float values -- don't try optimize speed
+
+static
+LPBYTE PackLabDouble(register _LPcmsTRANSFORM Info, register WORD wOut[], register LPBYTE output)
+{
+    cmsLabEncoded2Float((LPcmsCIELab) output, wOut);
+    output += sizeof(cmsCIELab);
+
+    return output;
+}
+
+static
+LPBYTE PackXYZDouble(register _LPcmsTRANSFORM Info, register WORD wOut[], register LPBYTE output)
+{
+    cmsXYZEncoded2Float((LPcmsCIEXYZ) output, wOut);
+    output += sizeof(cmsCIEXYZ);
+
+    return output;
+}
+
+static
+LPBYTE PackInkDouble(register _LPcmsTRANSFORM Info, register WORD wOut[], register LPBYTE output)
+{
+    double* Inks = (double*) output;
+    int nChan = T_CHANNELS(Info -> OutputFormat);
+    int i;
+
+    for (i=0; i <  nChan; i++) {
+
+        Inks[i] = wOut[i] /  655.35;
+    }
+
+    return output + (nChan + T_EXTRA(Info ->OutputFormat)) * sizeof(double);
+
+}
+
+
+static
+LPBYTE PackDouble(register _LPcmsTRANSFORM Info, register WORD wOut[], register LPBYTE output)
+{
+    double* Inks = (double*) output;
+    int nChan = T_CHANNELS(Info -> OutputFormat);
+    int i;
+
+    for (i=0; i <  nChan; i++) {
+
+        Inks[i] = wOut[i] /  65535.0;
+    }
+
+    return output + (nChan + T_EXTRA(Info ->OutputFormat)) * sizeof(double);
+
+}
+
+
 //  choose routine from Input identifier
 
-_cmsFIXFN _cmsIdentifyInputFormat(DWORD dwInput)
+_cmsFIXFN _cmsIdentifyInputFormat(_LPcmsTRANSFORM xform, DWORD dwInput)
 {
        _cmsFIXFN FromInput = NULL;
 
 
+       // Check Named Color
+
+       if (xform) {
+           
+           if (xform ->InputProfile) {
+
+           if (cmsGetDeviceClass(xform ->InputProfile) == icSigNamedColorClass) {
+
+                if (dwInput != TYPE_NAMED_COLOR_INDEX) {
+                    cmsSignalError(LCMS_ERRC_ABORTED, "Named color needs TYPE_NAMED_COLOR_INDEX");
+                    return NULL;
+                }
+           }
+                
+        }
+       }
+
+       // Unencoded modes
+
+       if (T_BYTES(dwInput) == 0) {
+
+           switch (T_COLORSPACE(dwInput)) {
+               
+           case PT_Lab: 
+                    FromInput = UnrollLabDouble;
+                    break;
+           case PT_XYZ: 
+                    FromInput = UnrollXYZDouble;
+                    break;
+
+           // 0.0 .. 1.0 range
+
+           case PT_GRAY:
+           case PT_RGB:
+           case PT_YCbCr:
+           case PT_YUV:
+           case PT_YUVK:
+           case PT_HSV:
+           case PT_HLS:
+           case PT_Yxy: 
+                    FromInput = UnrollDouble;
+                    break;
+
+            // Inks (%) 0.0 .. 100.0
+
+           default:
+                    FromInput = UnrollInkDouble;
+                    break;
+           }
+                    
+       }
+       else     
+       
        if (T_PLANAR(dwInput)) {
 
        switch (T_BYTES(dwInput)) {
 
-              case 1: FromInput = UnrollPlanarBytes;
+              case 1: 
+                      FromInput = UnrollPlanarBytes;
                       break;
 
-              case 2:if (T_ENDIAN16(dwInput))
+              case 2:
+                  
+                      if (T_ENDIAN16(dwInput))
                             FromInput = UnrollPlanarWordsBigEndian;
                       else
-                            FromInput = UnrollPlanarWords;
+                            FromInput = UnrollPlanarWords;                
                       break;
 
               default:;
@@ -1128,9 +1308,12 @@ _cmsFIXFN _cmsIdentifyInputFormat(DWORD dwInput)
                       }
                       break;
 
+              case 5:
               case 6:
+              case 7:
+              case 8:
                    if (!T_DOSWAP(dwInput) && !T_SWAPFIRST(dwInput))
-                       FromInput = Unroll6Bytes;
+                       FromInput = UnrollAnyBytes;
                    break;
 
                     
@@ -1205,9 +1388,12 @@ _cmsFIXFN _cmsIdentifyInputFormat(DWORD dwInput)
                       }
                       break;
 
+              case 5:
               case 6:
+              case 7:
+              case 8:
                     if (!T_DOSWAP(dwInput) && !T_SWAPFIRST(dwInput))
-                       FromInput = Unroll6Words;
+                       FromInput = UnrollAnyWords;
                     break;
 
               }
@@ -1225,10 +1411,44 @@ _cmsFIXFN _cmsIdentifyInputFormat(DWORD dwInput)
 
 //  choose routine from Input identifier
 
-_cmsFIXFN _cmsIdentifyOutputFormat(DWORD dwOutput)
+_cmsFIXFN _cmsIdentifyOutputFormat(_LPcmsTRANSFORM xform, DWORD dwOutput)
 {
        _cmsFIXFN ToOutput = NULL;
 
+
+       if (T_BYTES(dwOutput) == 0) {
+
+           switch (T_COLORSPACE(dwOutput)) {
+               
+           case PT_Lab: 
+                    ToOutput = PackLabDouble;
+                    break;
+           case PT_XYZ: 
+                    ToOutput = PackXYZDouble;
+                    break;
+
+           // 0.0 .. 1.0 range
+           case PT_GRAY:
+           case PT_RGB:
+           case PT_YCbCr:
+           case PT_YUV:
+           case PT_YUVK:
+           case PT_HSV:
+           case PT_HLS:
+           case PT_Yxy: 
+                    ToOutput = PackDouble;
+                    break;
+
+            // Inks (%) 0.0 .. 100.0
+
+           default:
+                    ToOutput = PackInkDouble;
+                    break;
+           }
+                    
+       }
+       else 
+
        if (T_PLANAR(dwOutput)) {
 
        switch (T_BYTES(dwOutput)) {
@@ -1447,6 +1667,7 @@ _cmsFIXFN _cmsIdentifyOutputFormat(DWORD dwOutput)
                              break;
 
 
+                     case 5:
                      case 7:
                      case 8:
                      case 9:
@@ -1487,3 +1708,44 @@ _cmsFIXFN _cmsIdentifyOutputFormat(DWORD dwOutput)
 
               return ToOutput;
 }
+
+// User formatters for (weird) cases not already included
+
+void LCMSEXPORT cmsSetUserFormatters(cmsHTRANSFORM hTransform, DWORD dwInput,  cmsFORMATTER Input,
+                                                               DWORD dwOutput, cmsFORMATTER Output)
+{
+    _LPcmsTRANSFORM xform = (_LPcmsTRANSFORM) (LPSTR) hTransform;
+    
+    xform ->FromInput = (_cmsFIXFN) Input;
+    xform ->InputFormat = dwInput;
+    xform ->ToOutput  = (_cmsFIXFN) Output;
+    xform ->OutputFormat = dwOutput;
+
+}
+
+void LCMSEXPORT cmsGetUserFormatters(cmsHTRANSFORM hTransform, 
+                                     LPDWORD InputFormat, cmsFORMATTER* Input, 
+                                     LPDWORD OutputFormat, cmsFORMATTER* Output)
+{
+    _LPcmsTRANSFORM xform = (_LPcmsTRANSFORM) (LPSTR) hTransform;
+    
+    *Input =  (cmsFORMATTER) xform ->FromInput;
+    *InputFormat = xform -> InputFormat;
+    *Output = (cmsFORMATTER) xform ->ToOutput;
+    *OutputFormat = xform -> OutputFormat;
+}
+
+
+// Change format of yet existing transform. No colorspace checking is performed
+
+void LCMSEXPORT cmsChangeBuffersFormat(cmsHTRANSFORM hTransform, 
+                                        DWORD dwInputFormat, 
+                                        DWORD dwOutputFormat)
+{
+
+    cmsSetUserFormatters(hTransform, 
+                        dwInputFormat,
+                        (cmsFORMATTER) _cmsIdentifyInputFormat(hTransform, dwInputFormat),
+                        dwOutputFormat,
+                        (cmsFORMATTER) _cmsIdentifyOutputFormat(hTransform, dwOutputFormat));
+}
