@@ -1,6 +1,6 @@
 //
 //  Little cms
-//  Copyright (C) 1998-1999 Marti Maria
+//  Copyright (C) 1998-2002 Marti Maria
 //
 // THIS SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND,
 // EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY
@@ -30,6 +30,8 @@
 
 // Test Suite for Little cms
 
+// #define ICM_COMPARATIVE		1
+// #define CHECK_SPEED			1
 
 #include "lcms.h"
 #include <time.h>
@@ -101,6 +103,10 @@ int CheckSwab(void)
 		return 1;
 #endif
 
+#ifdef USE_BIG_ENDIAN
+		return 1;
+#endif
+
 	swab(Test, Test, 6);
 
 	if (strncmp(Test, "\x2\x1\x4\x3\x6\x5", 6) != 0)
@@ -139,7 +145,7 @@ void PrintStatistics(clock_t atime, LPSTATS Stats)
        double a;
 
        diff = clock() - atime;
-       a = (double) diff / CLK_TCK;
+       a = (double) diff / CLOCKS_PER_SEC;
 
        // These are statistics of 16 bit, so divide
        // by 257 to get dE relative to 8 bits
@@ -261,7 +267,6 @@ int TestReversingOfCurves(void)
     double dE;
     STATS Stats;
 
-
     printf("Testing reversing of curves ...");
     ClearStats(&Stats);
 
@@ -290,6 +295,9 @@ int TestReversingOfCurves(void)
 
     PrintStatistics(0, &Stats); 
     printf(" pass.\n");
+	cmsFreeGamma(Gamma);
+	cmsFreeGamma(Reverse);
+	cmsFreeGamma(Computed);
     return 1;
 }
 
@@ -660,9 +668,9 @@ int Test3D(void)
 
                                               };
 
-
+    
       printf("Testing 3D interpolation on LUT...");
-
+      
       // 1.- Allocate an empty LUT
 
       MyLut = cmsAllocLUT();
@@ -811,7 +819,6 @@ BOOL TestMatrixCreation(void)
 }
 
 
-
 /*
 
        Used for debug purposes
@@ -868,7 +875,7 @@ double VecDist(Scanline_rgb2 *bin, Scanline_rgb2 *bout)
 // Green is most lightful, eye is most accurate on blue.
 
 static
-int TestFullSpectrum(cmsHTRANSFORM xform, int nRedInterv)
+int TestFullSpectrum(cmsHTRANSFORM xform, int nRedInterv, int MaxErr)
 {
        int r, g, b;
        double err;
@@ -918,11 +925,13 @@ int TestFullSpectrum(cmsHTRANSFORM xform, int nRedInterv)
                                           Stats.Peak = err;
 
 
-                                   if (err > 0x1000L)
+                                   if (err > MaxErr)
                                    {
                                           printf("Coarse error! : In=(%x,%x,%x) Out=(%x,%x,%x)\n",
                                                         bin[b].r, bin[b].g, bin[b].b,
                                                         bout[b].r, bout[b].g, bout[b].b);
+										  free(bin);
+										  free(bout);
                                           return 0;
                                    }
                             }
@@ -945,7 +954,7 @@ int TestFullSpectrum(cmsHTRANSFORM xform, int nRedInterv)
 static
 int TestInducedError(DWORD Type)
 {
-       cmsHPROFILE *In, *Out;
+       cmsHPROFILE In, Out;
        cmsHTRANSFORM xform;
        int nMaxError;
 
@@ -959,7 +968,7 @@ int TestInducedError(DWORD Type)
                                   INTENT_RELATIVE_COLORIMETRIC, 0);
               
 
-       nMaxError = TestFullSpectrum(xform, 31);
+       nMaxError = TestFullSpectrum(xform, 31, 0x1000L);
 
        printf("\n");
 
@@ -1124,7 +1133,7 @@ LPGAMMATABLE Build_sRGBGamma(void)
     Parameters[3] = 1. / 12.92;
     Parameters[4] = 0.04045;    // d
 
-    return cmsBuildParametricGamma(1024, 3, Parameters);
+    return cmsBuildParametricGamma(1024, 4, Parameters);
 }
 
 static
@@ -1280,9 +1289,7 @@ int RealProfilesTest(void)
        In2   = cmsOpenProfileFromFile("sRGBSpac.icm", "rb");
        Out2  = cmsCreateXYZProfile();
 
-       // printf("1=%s\n",  cmsTakeProductName(In1));
-       // printf("2=%s\n",  cmsTakeProductName(In2));
-
+      
        // Since LUT is 8-bits width,
        xform1 = cmsCreateTransform(In1, TYPE_RGBA_16, Out1, TYPE_XYZA_16, 0, cmsFLAGS_NOTPRECALC|cmsFLAGS_MATRIXINPUT);
        xform2 = cmsCreateTransform(In2, TYPE_RGBA_16, Out2, TYPE_XYZA_16, 0, cmsFLAGS_NOTPRECALC);
@@ -1320,9 +1327,9 @@ int TestPreview(void)
        Out   = cmsCreateLabProfile(NULL);
        Proof = cmsCreateLabProfile(NULL);
 
-       xform = cmsCreateProofingTransform(In, TYPE_LABA_16, Out, TYPE_LABA_16, Proof, 0, 0, 0);
+       xform = cmsCreateProofingTransform(In, TYPE_LABA_16, Out, TYPE_LABA_16, Proof, 0, 0, cmsFLAGS_SOFTPROOFING);
 
-       nMaxErr = TestFullSpectrum(xform, 31);
+       nMaxErr = TestFullSpectrum(xform, 31, 0x1000L);
 
        cmsDeleteTransform(xform);
        cmsCloseProfile(In);
@@ -1358,24 +1365,158 @@ int TestMultiprofile(void)
     Profiles[5] = hXYZ;
     Profiles[6] = hsRGB;
 
-    hXForm = cmsCreateMultiprofileTransform(Profiles, 7, TYPE_RGBA_16, TYPE_RGBA_16, INTENT_RELATIVE_COLORIMETRIC, 0);
+    hXForm = cmsCreateMultiprofileTransform(Profiles, 7, TYPE_RGBA_16, TYPE_RGBA_16, INTENT_RELATIVE_COLORIMETRIC, cmsFLAGS_HIGHRESPRECALC);
     
-    printf("Checking multiprofile transforms (6 profiles):");
+    printf("Checking multiprofile transforms (6 profiles)");
 
-    nMaxErr = TestFullSpectrum(hXForm, 31);
+    nMaxErr = TestFullSpectrum(hXForm, 31, 0x1000L);
 
     cmsDeleteTransform(hXForm);
     cmsCloseProfile(hsRGB);
     cmsCloseProfile(hXYZ);
     cmsCloseProfile(hLab);
 
-    if (!nMaxErr) printf("pass.\n");
+    printf("\n");
 
     return nMaxErr;
-
-
 }
 
+// Check linearization and other goodies
+
+static
+int TestLinearizationDevicelink()
+{
+	LPGAMMATABLE Transfer[3];
+	cmsHPROFILE hLin1, hLin2;
+    cmsHTRANSFORM hXForm;
+    cmsHPROFILE Profiles[10];
+    int nMaxErr;
+
+	printf("Checking linearization devicelink");
+
+	Transfer[0] = cmsBuildGamma(256, 1./2.2);
+	Transfer[1] = cmsBuildGamma(256, 1./2.2);
+	Transfer[2] = cmsBuildGamma(256, 1./2.2);
+
+	hLin1 = cmsCreateLinearizationDeviceLink(icSigRgbData, Transfer);
+
+	cmsFreeGammaTriple(Transfer);
+	
+
+
+	Transfer[0] = cmsBuildGamma(256, 2.2);
+	Transfer[1] = cmsBuildGamma(256, 2.2);
+	Transfer[2] = cmsBuildGamma(256, 2.2);
+
+	hLin2 = cmsCreateLinearizationDeviceLink(icSigRgbData, Transfer);
+
+	cmsFreeGammaTriple(Transfer);
+
+	Profiles[0] = hLin1;
+	Profiles[1] = hLin2;
+
+	hXForm = cmsCreateMultiprofileTransform(Profiles, 2, TYPE_RGBA_16, TYPE_RGBA_16, INTENT_RELATIVE_COLORIMETRIC, cmsFLAGS_HIGHRESPRECALC);
+	if (!hXForm) {
+
+		printf("Error!\n");
+		return 1;
+	}
+	
+
+    nMaxErr = TestFullSpectrum(hXForm, 31, 0x1000L);
+
+	cmsDeleteTransform(hXForm);
+    cmsCloseProfile(hLin1);
+    cmsCloseProfile(hLin2);
+
+	printf("\n");
+
+    return nMaxErr;
+}
+
+static
+int TestInkLimiting()
+{
+    cmsHPROFILE hIL;
+    cmsHTRANSFORM hXForm;
+    BYTE In[4], Out[4];
+    int i, j, k, l;
+
+    printf("Testing ink limiting ");
+    
+    hIL = cmsCreateInkLimitingDeviceLink(icSigCmykData, 100);
+
+    
+    hXForm = cmsCreateTransform(hIL, TYPE_CMYK_8, NULL, TYPE_CMYK_8, INTENT_RELATIVE_COLORIMETRIC, 0);
+    if (!hXForm) {
+
+        printf("Error!\n");
+        return 0;
+    }
+    
+    for (l=0; l < 255; l += 8) {
+        Dot();      
+        for (k=0; k < 255; k += 8) 
+            for (j=0; j < 255; j += 8) 
+                for (i=0; i < 255; i += 8) {
+
+                    In[0] = i; In[1] = j; In[2] = k; In[3] = l;
+
+                    cmsDoTransform(hXForm, In, Out, 1);
+
+                    if (Out[0] + Out[1] + Out[2] + Out[3] > 0x100) {
+            
+                        printf("Failed!\n");
+                        return 0;   
+                    }
+        }
+    }
+
+    cmsDeleteTransform(hXForm);
+    cmsCloseProfile(hIL);
+    printf("pass.\n");
+
+    return 1;
+}
+
+
+
+static
+int TestDeviceLinkGeneration()
+{
+    cmsHTRANSFORM hXForm, hIdentity;
+	cmsHPROFILE hDevLink, hsRGB;
+    int nMaxErr;
+
+
+	printf("Checking devicelink generation");
+
+	hsRGB	  = cmsOpenProfileFromFile("sRGB Color Space Profile.icm", "r");
+	hIdentity = cmsCreateTransform(hsRGB, TYPE_RGBA_16, hsRGB, TYPE_RGBA_16, INTENT_RELATIVE_COLORIMETRIC, 0);
+	hDevLink  = cmsTransform2DeviceLink(hIdentity, 0);
+	_cmsSaveProfile(hDevLink, "devicelink.icm");
+
+	cmsCloseProfile(hDevLink);
+	cmsCloseProfile(hsRGB);
+	cmsDeleteTransform(hIdentity);
+
+	hDevLink = cmsOpenProfileFromFile("devicelink.icm", "r");
+	hXForm   = cmsCreateTransform(hDevLink, TYPE_RGBA_16, NULL, TYPE_RGBA_16, INTENT_RELATIVE_COLORIMETRIC, 0);	
+    nMaxErr  = TestFullSpectrum(hXForm, 31, 0x1000L);
+
+	cmsDeleteTransform(hXForm);	
+    cmsCloseProfile(hDevLink);
+    
+	printf("\n");
+	unlink("devicelink.icm");
+
+    return nMaxErr;
+}
+
+
+
+
+
 
 static
 void CheckPlanar(void)
@@ -1405,7 +1546,7 @@ void CheckPlanar(void)
         cmsCloseProfile(hsRGB);
 }
 
-
+#ifdef ICM_COMPARATIVE
 #ifndef NON_WINDOWS
 
 static
@@ -1462,7 +1603,7 @@ void CompareWithICM(void)
     }
 
     diff = clock() - atime;
-    seconds = (double) diff / CLK_TCK;
+    seconds = (double) diff / CLOCKS_PER_SEC;
 
 
     printf("done. [%d tics, %g sec.]\n", (int) diff, seconds);
@@ -1492,7 +1633,7 @@ void CompareWithICM(void)
     }
 
     diff = clock() - atime;
-    seconds = (double) diff / CLK_TCK;
+    seconds = (double) diff / CLOCKS_PER_SEC;
 
     printf("done. [%d tics, %g sec.]\n", (int) diff, seconds);
 
@@ -1503,14 +1644,63 @@ void CompareWithICM(void)
 }
 
 #endif
+#endif
 
 
+#ifdef CHECK_SPEED
+static
+void SpeedTest(void)
+{
+
+    int r, g, b;
+    clock_t atime;
+    double seconds, diff;
+    cmsHPROFILE hlcmsProfileIn, hlcmsProfileOut;
+    cmsHTRANSFORM hlcmsxform;
+	COLOR In, Out;
+   
+   
+	printf("\n\nRaw speed check:\n");
+
+    hlcmsProfileIn  = cmsOpenProfileFromFile("sRGB Color Space Profile.ICM", "r");
+    hlcmsProfileOut = cmsOpenProfileFromFile("sRGB Color Space Profile.ICM", "r");
+
+    hlcmsxform  = cmsCreateTransform(hlcmsProfileIn, TYPE_RGB_16, hlcmsProfileOut, TYPE_RGB_16, INTENT_PERCEPTUAL, 0);
+
+    printf("lcms is transforming full spectrum...");
+
+    atime = clock();
+
+    for (r=0; r < 255; r++)
+        for (g=0; g < 255; g++)
+            for (b=0; b < 255; b++) {
+
+        In.rgb.red   = (r << 8) | r;
+        In.rgb.green = (g << 8) | g;
+        In.rgb.blue  = (b << 8) | b;
+
+        cmsDoTransform(hlcmsxform, &In.rgb, &Out.rgb, 1);
+    }
+
+    diff = clock() - atime;
+    seconds = (double) diff / CLOCKS_PER_SEC;
+	
+    printf("done.\n[%d tics, %g sec, %g Mb/sec.]\n", (int) diff, seconds, 32. / seconds );
+
+    cmsDeleteTransform(hlcmsxform);
+    cmsCloseProfile(hlcmsProfileIn);
+    cmsCloseProfile(hlcmsProfileOut);
+
+}
+
+#endif
+
 
 int main(int argc, char *argv[])
 {
-       int lExhaustive = 0;
-       
-       printf("little cms testbed. Ver 1.09 [build %s %s]\n\n", __DATE__, __TIME__);
+       int lExhaustive = 0;	  
+
+       printf("little cms testbed. Ver 1.11 [build %s %s]\n\n", __DATE__, __TIME__);
       
 #ifndef LCMS_DLL
 
@@ -1518,11 +1708,15 @@ int main(int argc, char *argv[])
 	   if (!CheckSwab()) return 1;
 
        TestFixedPoint();
-       if (!TestFixedScaling()) return 1;     
+	 
+       if (!TestFixedScaling()) return 1;     	   
        if (!TestJointCurves()) return 1;
-       if (!TestReversingOfCurves()) return 1;
+	   if (!TestReversingOfCurves()) return 1;
        if (!TestLinearInterpolation(lExhaustive)) return 1;
        if (!TestReverseLinearInterpolation()) return 1;
+
+	   
+
        if (!Test3D()) return 1;
        if (!TestMatrixCreation()) return 1;
        if (!GetInfoTest()) return 1;
@@ -1535,13 +1729,24 @@ int main(int argc, char *argv[])
 
        if (!TestPreview()) return 1;
        if (!TestMultiprofile()) return 1;
+	   if (!TestLinearizationDevicelink()) return 1;
+       if (!TestInkLimiting()) return 1;
+	   if (!TestDeviceLinkGeneration()) return 1;
 
+#ifdef ICM_COMPARATIVE
 #ifndef NON_WINDOWS
        CompareWithICM();
 #endif
+#endif
+
+#ifdef CHECK_SPEED 	  
+	   SpeedTest();
+#endif
 
        printf("\nSuccess.\n");
 
        return 0;
 
 }
+
+
